function delab( varargin )

% DELab is a GUI for studying differential equations.
%
% DELab accesses Matlab's symbolic, numerical, and graphical tools.
% In addition to DELab's built-in capabilities, many of the results
% of its numerical and symbolic work can be returned to the Matlab
% command window for further analysis; e.g., to compare an analytic
% solution formula with an array of values from a numeric solver.
% In a similar spirit, graphical output can be customized using
% Matlab's plot editing tools.
%
% To start DELab, type "delab" in the Matlab command window. From
% the DELab menu bar, select Equation to identify a type of
% differential equation, then choose an analytic, numerical, or
% graphical tool from the menu bar in the equation window.  Use the
% corresponding tool window to control or refine the action of the
% tool; e.g., to create a Matlab in-line function from an analytic
% solution.
%
% Edit variables, functions, parameter names and values, initial
% and other data as desired.  On Windows systems, TOUCH the ENTER
% key after each edit.  Use sqrt(), abs(), cos(), sin(), tan(),...
% for square root, absolute value, sine, cosine, tangent, ...  Use
% *, /, ^ for multiplication, division, exponentiation.
%
% See also ABOUT, CONTENTS, TEXTBOOK help

% Copyright 1999 Paul Davis.
% For nonprofit educational use only

% Set of actions for main DELab window
S = { 'order1', 'order2', 'system2', 'systemN',...
   'Laplace', 'Fourier', 'AnlEvalue',...
   'EvalFunc', 'NumEvalue',...
   'PlotFunc', 'ShowEig',...
   'HelpLocal', 'HelpText',...
   'quit' };
% The function that governs action foo is f_foo.
% The function that opens the GUI window for action foo is w_foo.

switch nargin
case 0
   action = 'initialize';
case 1
   action = varargin{1};
end

switch action
case { 'initialize' '1' '2' 'sys2' 'sysN' 'sysn' }
   if ~exist( 'o1list.mat', 'file' ), order1list; end
   if ~exist( 'o2list.mat', 'file' ), order2list; end
   if ~exist( 'o1sys2list.mat', 'file' ), order1sys2list; end
   if ~exist( 'o1sysNlist.mat', 'file' ), order1sysNlist; end
   delabwinH = w_delab;

   switch action
   case '1'
      f_order1( 'initialize_scalar', '1' );
      % 1 is number of default equation

   case '2'
      f_order2( 'initialize', '2' );

   case 'sys2'
      f_order1( 'initialize_sys2', '10' );

   case 'sysN'
      f_order1( 'initialize_sysN', '2' );

   end  % switch

case 'eigshow'
   w_entermatrix( 2, 'eigshow' );

case { 'AnlEvalue' 'NumEvalue' }
   prompt = [ 'How many rows/columns in the matrix?' ];
   title = [ 'Number of rows/columns' ];
   def = { num2str( 2 ) };
   answer = inputdlg( prompt, title, [ 1 ( 7 + length( prompt ) ) ], def );

   if ~isempty( answer )
      n = str2num( answer{1} );
   else
      return;
   end

   w_entermatrix( n, action );

case { 'starteigshow' 'startAnlEvalue' 'startNumEvalue' }
   pud = get( gcbf, 'UserData' );
   [n, n] = size( pud.entry );
   cellmatrix = cell( n*n, 1 );
   nummatrix = zeros(n, n);

   for j = 1:n
      for i = 1:n
         s = get( pud.Handle.entry(i, j), 'String' );
         if isempty( deblank( s ) )
            errordlg( 'Provide an entry for every coefficient.',...
               'Error' );
            return
         end  % if

         cellmatrix{ i + n*(j - 1) } = char( s );

         if isletter( s ) & ...
            ( strcmp( action, 'starteigshow' ) | ...
               strcmp( action, 'startNumEvalue' ) )
            errordlg( 'Enter numerical values only; no symbols permitted.',...
               'Error' );
            return;
         elseif ~isletter( s )
            nummatrix(i, j) = str2num( s );
         end  % if

      end  % for i
   end  % for j

   switch action
   case 'starteigshow'
      figure;
      eigshow( nummatrix );

   case { 'startNumEvalue' 'startAnlEvalue' }

      switch action
      case 'startAnlEvalue'

         for j = 1:n
            for i = 1:n
               matrix(i, j) = sym( cellmatrix{i + n*(j - 1)} );
            end  % i
         end  % j

         [ V, D ] = eig( matrix );
         V = V(:);
         D = diag( D ); D = D(:);

         cellV = cell( n*n, 1 );
         cellD = cell( n, 1 );

         for i = 1:n*n
            cellV{i} = V(i);
         end  % for

         for i = 1:n
            cellD{i} = D(i);
         end  % for

         w_evalue( n, cellmatrix, cellV, cellD, 'Analytic eigenvalues' );

      case 'startNumEvalue'
         [ V, D ] = eig( nummatrix );
         V = V(:);
         D = diag( D ); D = D(:);
         cellV = cell( n*n, 1 );
         cellD = cell( n, 1 );

         for i = 1:n*n
            cellV{i} = num2str( V(i) );
         end  % for

         for i = 1:n
            cellD{i} = num2str( D(i) );
         end  % for

         w_evalue( n, cellmatrix, cellV, cellD, 'Numerical eigenvalues' );

         end  % switch

   end  % switch

case 'Laplace'
   sorry;
case 'Fourier'
   sorry;
case 'EvalFunc'
   sorry;
case 'ShowEig'
   sorry;
case 'HelpLocal'
   sorry;
case 'HelpText'
   sorry;
case 'quit'
   close all;
otherwise
   % Default to 'initialize'.
   warndlg( 'Input argument to DELab ignored.', 'Warning' );
   delabwinH = w_delab;
end

% end delab
