function order1H = f_order1( varargin )

% Callback functions for order 1 ODE window in DELab
% Useage: f_order1 action number

% An equation dy/dt = f is defined by a structure with the following
% fields.  All (including parameter values pvalue) are character
% strings except numeqn and nump; y, f, and y0 are cell arrays of
% character strings.
%
% For equation lists: name (optional)
%
% For the equation itself: y, numeqn (= length( y )), t, f, pname,
% pvalue, nump (= number of parameter boxes in current equation
% window)
%
% For IVP: t0, y0
%
% For plotting and numerical methods: tf, deltat, NumStep, ymin,
% ymax, method

% Appears must make all assignments to cell structures using strings
% via cell index, else lose cell class if cell is one dimensional


% Set of actions
S = { 'FromList', 'Save',...         % w_order1 menu ===
   'AnlSolveODE', 'AnlSolveIVP', 'StdyState',...
   'numivp', 'visdemo',...
   'Dfield', 'PlotIVP',...
   'HelpLocal', 'HelpText',...
   'ok', 'close', 'quit'...          % push button selections
   'y_edit', 't_edit', 'f_edit',...  % edit box actions
   'pname_edit', 'pvalue_edit',...
   't0_edit', 'y0_edit', ...
   'Euler', 'Heun', 'RK4',...        % w_numcntrl menu ===
   'ode23', 'ode45', 'ode113',...
   'ode23s', 'ode15s'...
   'NumHelp',...,
   'start', 'stop',...                % push buttons
   'tf_edit', 'deltat_edit',...       % edit boxes
   'NumStep_edit', 'ymin_edit'...
   'ymax_edit',...
   'StepSize_radio', 'NumStep_radio',... % radio buttons
   'UseEnteredIC_radio', 'ClickIC_radio',...
   'NewFig_radio', 'CurrentFig_radio',...
   'dfield_check', 'high_check',...    % check boxes
    'yaxis_check' };

switch nargin
case 0
   errordlg( 'No argument given to f_order1', 'Error' );
%   action = 'initialize';
case 1
   action = varargin{1};
   inputnumber = -1;
case 2
   action = varargin{1};
   inputnumber = str2num( varargin{2} );
      % e.g., Equation from default list
case 3
   action = varargin{1};
   inputnumber = -1;  % varargin{2}
   inputDE = varargin{3};  % input system as structure
end

switch action
case { 'initialize_scalar' 'initialize_sys2' ...
   'initialize_sysN' }
   % These colors not used yet
   lightgray = [ 0.8 0.8 0.8 ];
   darkgray = [0.752941176470588 0.752941176470588 0.752941176470588];
   white = [ 1 1 1 ];
   black = [ 0 0 0 ];

   eqn = inputnumber;

   switch action
   case 'initialize_scalar'
      % 1st-order scalar ode
      load o1list;
   case 'initialize_sys2'
      load o1sys2list;
      % load d:\matlab5\toolbox\matlab\delab_dev\o1sys2list;
   case 'initialize_sysN'
      load o1sysNlist;
   end   % switch action

   if eqn > 0 & nargin == 2 % Use a default from list
      % Describe the IVP
      ud.y = defaultlist{eqn}.y;
      ud.numeqn = length( ud.y );
      ud.t = defaultlist{eqn}.t;
      ud.f = defaultlist{eqn}.f;
      ud.pname = defaultlist{eqn}.pname;
      ud.pvalue = defaultlist{eqn}.pvalue;
      ud.nump = length( ud.pname );
      ud.t0 = defaultlist{eqn}.t0;
      ud.y0 = defaultlist{eqn}.y0;

      % Defaults for various control panels
      ud.tf = defaultlist{eqn}.tf;
      ud.deltat = defaultlist{eqn}.deltat;
      ud.NumStep = defaultlist{eqn}.NumStep;

      if isempty( defaultlist{eqn}.ymin )
         ud.ymin = { num2str( -2*str2num( ud.y0{1} ) ); num2str( -2*str2num( ud.y0{2} ) ) };
      else
         ud.ymin = defaultlist{eqn}.ymin;
      end

      if isempty( defaultlist{eqn}.ymax )
         ud.ymax = { num2str( 2*str2num( ud.y0{1} )) ; num2str( 2*str2num( ud.y0{2} ) ) };
      else
         ud.ymax = defaultlist{eqn}.ymax;
      end

      ud.method = defaultlist{eqn}.method;
   elseif eqn == 0 & nargin == 2
      % open blank equation window
      prompt = [ 'How many equations in the system?' ];
      title = [ 'Number of equations' ];
      def = { num2str( 3 ) };
      answer = inputdlg( prompt, title, [ 1 ( 7 + length( prompt ) ) ], def );

      if ~isempty( answer )
         neqn = str2num( answer{1} );
      else
         return;
      end

      ud.y = cell(neqn, 1);
      ud.numeqn = length( ud.y );
      ud.t = '';
      ud.f = cell(neqn, 1);
      ud.pname = cell(2, 1);
      ud.pvalue = cell(2, 1);
      ud.nump = length( ud.pname );
      ud.t0 = '';
      ud.y0 = cell(neqn,1);

      % Defaults for various control panels
      ud.tf = '';
      ud.deltat = '';
      ud.NumStep = '';
      ud.ymin = cell(neqn, 1);
      ud.ymax = cell(neqn, 1);

      ud.method = 'rk45';
   elseif eqn < 0 & nargin == 3
      % ud was pre-set in inputDE, probably with equiv 1st order sys
      ud = inputDE;

      % Defaults for various control panels
      if ~exist( ud.tf ),  ud.tf = '10';  end
      if ~exist( ud.deltat ),  ud.deltat = '1';  end
      if ~exist( ud.NumStep ),  ud.NumStep = '10';  end

      if ~exist( inputDE.ymin ) | isempty( inputDE.ymin )
         ud.ymin = cell(2, 1);
         ud.ymin{1} = num2str( -2*str2num( ud.y0{1} ) );
         ud.ymin{2} = num2str( -2*str2num( ud.y0{2} ) );
         if ud.ymin{1} == '0'  ud.ymin{1} = '-1';  end
         if ud.ymin{2} == '0'  ud.ymin{2} = '-1';  end
      end

      if ~exist( inputDE.ymax ) | isempty( inputDE.ymax )
         ud.ymax = cell(2, 1);
         ud.ymax{1} = num2str( 2*str2num( ud.y0{1} ) );
         ud.ymax{2} = num2str( 2*str2num( ud.y0{2} ) );
         if ud.ymax{1} == '0'  ud.ymax{1} = '1';  end
         if ud.ymax{2} == '0'  ud.ymax{2} = '1';  end
      end

      if ~exist( ud.method ) | isempty( ud.method )
         ud.method = 'RK4';
      end

   end  % if

   % pre-allocate storage for control panel figure handles
   ud.Handle.order1 = -1;
   ud.Handle.numivp = -1;
   ud.Handle.visdemo = -1;
   ud.Handle.dfield = -1;
   ud.Handle.phaseline = -1;
   ud.Handle.plotvst = -1;
   ud.Handle.phaseplane = -1;
   ud.Handle.phasespace = -1;
%   ud.Handle.plottedfigs = [];  % Figures available for plotting

   h = w_order1( ud );
   order1H = h;
   set( h, 'HandleVisibility', 'CallBack' );
   % ud.Handle.order1 = h
   % Do not set UserData to ud here; seems to freeze structure.
   % Instead, pass ud as arg of w_order1.

case 'FromDefaultList'
   ud = get( gcbf, 'Userdata' );
   oldnump = ud.nump;

   if ud.numeqn == 1
      load o1list.mat;
   elseif ud.numeqn == 2
      load o1sys2list.mat;
   else
      load o1sysNlist.mat;
   end

   eqn = inputnumber;

   new_neqn = length( defaultlist{eqn}.y );

   if ud.numeqn ~= new_neqn
      % different number of equations; new eqn window
      close( gcbf );

      if ud.Handle.numivp > 0, close( ud.Handle.numivp ); end
      if ud.Handle.plotvst > 0, close( ud.Handle.plotvst ); end
      if ud.Handle.phaseplane > 0, close( ud.Handle.phaseplane ); end
      if ud.Handle.phasespace > 0, close( ud.Handle.phasespace ); end

      if new_neqn == 2
         string = 'initialize_sys2';
      elseif new_neqn > 2
         string = 'initialize_sysN';
      end  % if

      f_order1( string, num2str( eqn ) );
      return;
   end  % if

   ud.y = defaultlist{eqn}.y;
   ud.t = defaultlist{eqn}.t;
   ud.f = defaultlist{eqn}.f;
   ud.pname = defaultlist{eqn}.pname;
   ud.pvalue = defaultlist{eqn}.pvalue;
   ud.nump = length( ud.pname );
   ud.t0 = defaultlist{eqn}.t0;
   ud.y0 = defaultlist{eqn}.y0;
   ud.tf = defaultlist{eqn}.tf;
   ud.deltat = defaultlist{eqn}.deltat;
   ud.NumStep = defaultlist{eqn}.NumStep;

   if isempty( defaultlist{eqn}.ymin )
      ud.ymin = { num2str( -2*abs( str2num( ud.y0{1} ) ) ); num2str( -2*abs( str2num( ud.y0{2} ) ) ) };
   else
      ud.ymin = defaultlist{eqn}.ymin;
   end

   if isempty( defaultlist{eqn}.ymax )
      ud.ymax = { num2str( 2*abs( str2num( ud.y0{1} ) ) ) ; num2str( 2*abs( str2num( ud.y0{2} ) ) ) };
   else
      ud.ymax = defaultlist{eqn}.ymax;
   end

   ud.method = defaultlist{eqn}.method;

   % ode solver output options
   ud.odeoptions = '';

   set( gcbf, 'Userdata', ud );

   for i = 1:ud.numeqn
      set( ud.Handle.y_box(i), 'String', ud.y{i} );
      set( ud.Handle.f_box(i), 'String', ud.f{i} );
      set( ud.Handle.yparen_box(i), 'String', [ char(ud.y{i}) '(' ] );
      set( ud.Handle.y0_box(i), 'String', ud.y0{i} );
   end;

   set( ud.Handle.t_box, 'String', ud.t );
   set( ud.Handle.t0_box, 'String', ud.t0 );

   if oldnump == ud.nump

      for i = 1:oldnump
         set( ud.Handle.pname_box(i), 'String', ud.pname{i} );
         set( ud.Handle.pvalue_box(i), 'String', ud.pvalue{i} );
         % Why can't assign cell arrays here?
      end;

      set( gcbf, 'UserData', ud );
   elseif oldnump < ud.nump

      % Replace equation window with one having
      % more parameter boxes.
      close( gcbf );
      h = w_order1( ud );
      set( h, 'HandleVisibility', 'CallBack' );
      ud = get( h, 'UserData' );
   else
%     oldnump > ud.nump

      % Use needed parameter boxes; blank the rest.
      % Add additional empty parameters.
      for i = 1:ud.nump
         set( ud.Handle.pname_box(i), 'String', ud.pname{i} );
         set( ud.Handle.pvalue_box(i), 'String', ud.pvalue{i} );
      end;

      for i = (ud.nump + 1):oldnump
         set( ud.Handle.pname_box(i), 'String', '' );
         set( ud.Handle.pvalue_box(i), 'String', '' );
      end;

      ud.nump = oldnump;
      set( gcbf, 'UserData', ud );
   end;

   % Restart any open control panels
   if ud.Handle.numivp > 0
      close( ud.Handle.numivp );
      ud.Handle.numivp = -1;
      set( ud.Handle.order1, 'UserData', ud );
      f_order1( 'numivp', num2str( ud.Handle.order1 ) );
   end

   if ud.Handle.visdemo > 0
      close( ud.Handle.visdemo );
      ud.Handle.visdemo = -1;
      set( ud.Handle.order1, 'UserData', ud );
      f_order1( 'visdemo', num2str( ud.Handle.order1 ) );
   end

   if ud.Handle.dfield > 0
      close( ud.Handle.dfield );
      ud.Handle.dfield = -1;
      set( ud.Handle.order1, 'UserData', ud );
      f_order1( 'dfield', num2str( ud.Handle.order1 ) );
   end

   if ud.Handle.phaseline > 0
      close( ud.Handle.phaseline );
      ud.Handle.phaseline = -1;
      set( ud.Handle.order1, 'UserData', ud );
      f_order1( 'phaseline', num2str( ud.Handle.order1 ) );
   end

   if ud.Handle.plotvst > 0
      close( ud.Handle.plotvst );
      ud.Handle.plotvst = -1;
      set( ud.Handle.order1, 'UserData', ud );
      f_order1( 'plotvst', num2str( ud.Handle.order1 ) );
   end

   if ud.Handle.phaseplane > 0
      close( ud.Handle.phaseplane );
      ud.Handle.phaseplane = -1;
      set( ud.Handle.order1, 'UserData', ud );
      f_order1( 'phaseplane', num2str( ud.Handle.order1 ) );
   end;

   if ud.Handle.phasespace > 0
      close( ud.Handle.phasespace );
      ud.Handle.phasespace = -1;
      set( ud.Handle.order1, 'UserData', ud );
      f_order1( 'phasespace', num2str( ud.Handle.order1 ) );
   end;

case 'Save'
   sorry;

case 'AnlSolveODE'
   set( gcf, 'Pointer', 'watch' );
   ud = get( gcbf, 'Userdata' );
   fstring = prm2val( ud.f, ud.pname, ud.pvalue );

   solvestr = [ 'D' ud.y{1} ' = ' fstring{1} ];
   for i = 2:length( ud.y )
      solvestr = [ solvestr ', D' ud.y{i} ' = ' fstring{i} ];
   end

   if badfunc( solvestr )
      set( gcbf, 'Pointer', 'arrow' );
      return;
   end

   try
      lastwarn( '' );
      implicit = 0;
      s = dsolve( solvestr, ud.t );

      if ~isempty( lastwarn )
         % Implicit solution, most likely
         implicit = 1;
         warndlg( lastwarn, 'Warning' );
         lastwarn( '' );
      end

   catch

      s = '';
      if ~isempty( lastwarn )
         warndlg( lastwarn, 'Warning' );
         lastwarn( '' );
      elseif ~isempty( lasterr )
         errordlg( lasterr, 'Error' );
         set( gcf, 'Pointer', 'arrow' );
         return;
      else
         warndlg( 'No compact analytic solution or faulty problem formulation', 'Warning' );
         set( gcbf, 'Pointer', 'arrow' );
         return;
      end  % if

   end  % try

   if ~isempty( s )
      solution = dsolve2cell( s, ud.y );
      multsoln = length( solution )/ud.numeqn;
      set( gcbf, 'Pointer', 'arrow' );

      if implicit
         f_anlsoln( 'asis', 'ide1', multsoln, solution );
      else
         f_anlsoln( 'asis', 'de1', multsoln, solution );
      end  % if

   else
      warndlg( 'No explicit solution could be found.', 'Warning' );
      set( gcbf, 'Pointer', 'arrow' );
      return;
   end

case 'AnlSolveIVP'
   set( gcf, 'Pointer', 'watch' );
   ud = get( gcbf, 'Userdata' );
   fstring = prm2val( ud.f, ud.pname, ud.pvalue );

   solvestr = [ 'D' ud.y{1} ' = ' fstring{1} ];
   icstring = [ ud.y{1} '(' ud.t0 ') = ' ud.y0{1} ];
   for i = 2:length( ud.y )
      solvestr = [ solvestr ', D' ud.y{i} ' = ' fstring{i} ];
      string = [ ud.y{i} '(' ud.t0 ') = ' ud.y0{i} ];
      icstring = [ icstring ', ' string ];
   end

   solvestr = [ solvestr ', ' icstring ];

   if badfunc( solvestr )
      set( gcbf, 'Pointer', 'arrow' );
      return;
   end

   try
      lastwarn( '' );
      implicit = 0;
      s = dsolve( solvestr, ud.t );

      if ~isempty( lastwarn )
         % Implicit solution likely
         implicit = 1;
         warndlg( lastwarn, 'Warning' );
         lastwarn( '' );
      end

   catch

      s = '';
      if ~isempty( lastwarn )
         warndlg( lastwarn, 'Warning' );
         lastwarn( '' );
      elseif ~isempty( lasterr )
         errordlg( lasterr, 'Error' );
         set( gcf, 'Pointer', 'arrow' );
         return;
      else
         warndlg( 'No compact analytic solution or faulty problem formulation', 'Warning' );
         set( gcbf, 'Pointer', 'arrow' );
         return;
      end  % if

   end

   if ~isempty( s )
      solution = dsolve2cell( s, ud.y );
      multsoln = length( solution )/ud.numeqn;
      set( gcbf, 'Pointer', 'arrow' );

      if implicit
         f_anlsoln( 'asis', 'iivp1', multsoln, solution );
      else
         f_anlsoln( 'asis', 'ivp1', multsoln, solution );
      end  % if

   else
      warndlg( 'No explicit solution could be found.', 'Warning' );
      set( gcbf, 'Pointer', 'arrow' );
      return;
   end

case 'StdyState'
   set( gcf, 'Pointer', 'watch' );
   ud = get( gcbf, 'Userdata' );
   neqn = length( ud.f );
   fstring = prm2val( ud.f, ud.pname, ud.pvalue );

   % Really autonomous?
   if neqn == 1
      string = 'equation';
   else
      string = 'system';
   end  % if

   for i = 1:neqn
      if char( diff( fstring{i}, ud.t ) ) ~= '0'
         set( gcf, 'Pointer', 'arrow' );
         errordlg( ['This ' string ' is not autonomous: ' ud.t ' appears explicitly in it.' ],'Error' );
         set( gcf, 'Pointer', 'arrow' );
         return;
      end
   end

   % Find steady states.
   quote = '''';
   solvestr = sprintf( '%s%s = 0%s', quote, fstring{1}, quote );
   varstr = sprintf( '%s%s%s', quote, char( ud.y{1} ), quote );
   for i = 2:neqn
      string = sprintf( '%s%s = 0%s', quote, fstring{i}, quote );
      solvestr = [ solvestr ', ' string ];
      string = sprintf( '%s%s%s', quote, char( ud.y{i} ), quote );
      varstr = [ varstr ', ' string ];
   end

   if badfunc( solvestr )
      set( gcbf, 'Pointer', 'arrow' );
      return;
   end

   try
       lastwarn( '' );

       if neqn == 1
          % can't find 2 solns of a*P - s*P^2 = 0 without
          % this case
           s = solve( solvestr, ud.y{1} );
       else
           solvstr = [ solvestr ', ' varstr ];
%           s = solve( solvestr )
         s = feval( 'solve', solvestr );
       end  % if

       if ~isempty( lastwarn )
          warndlg( lastwarn, 'Warning' );
          lastwarn( '' );
       end

   catch

      s = '';
      if ~isempty( lastwarn )
         warndlg( lastwarn, 'Warning' );
         lastwarn( '' );
      elseif ~isempty( lasterr )
         errordlg( lasterr, 'Error' );
         set( gcf, 'Pointer', 'arrow' );
         return;
      else
         warndlg( 'No explicit steady state could be found.', 'Warning' );
         set( gcbf, 'Pointer', 'arrow' );
         return;
      end  % if

   end

   if ~isempty( s )
      solution = dsolve2cell( s, ud.y );
      multsoln = length( solution )/ud.numeqn;
      set( gcbf, 'Pointer', 'arrow' );

      if ceil( multsoln ) ~= multsoln  |  multsoln == 0
          % Strange result returned by solve.
          warndlg( 'Attempt to find steady state failed.',...
              'Warning' );
          set( gcf, 'Pointer', 'arrow' );
          return;
      end  % if

      f_anlsoln( 'asis', 'ss1', multsoln, solution );
   else
      warndlg( 'No explicit solution could be found.', 'Warning' );
      set( gcbf, 'Pointer', 'arrow' );
      return;
   end

case { 'numivp' 'phaseline' 'dfield' 'visdemo' 'plotvst' ...
   'phaseplane' 'phasespace' } % Open control panel

   if inputnumber > 0
      hin = inputnumber;  % f_order1 has called itself
   else
      hin = gcbf;
   end;

   ud = get( hin, 'Userdata' );
   fieldname = action;
   codename = [ 'w_' action ]; % e.g., w_phaseplane

   % Autonomous for phaseline, phaseplane, phasespace?
   switch action
   case 'phaseline'
      fstring = prm2val( ud.f, ud.pname, ud.pvalue, ud.y );
      if char( diff( fstring{1}, ud.t ) ) ~= '0'
         errordlg( ['This equation is not autonomous; no phase line is available.' ],'Error' );
         return;
      end

   case { 'phaseplane' 'phasespace' }
      fstring = prm2val( ud.f, ud.pname, ud.pvalue, ud.y );
      for i = 1:length( ud.f )
         if char( diff( fstring{i}, ud.t ) ) ~= '0'
            errordlg( ['This system is not autonomous; no phase diagram is available.' ],'Error' );
            return;
         end
      end

   end  % switch

   panelhandle = getfield( ud.Handle, fieldname );
   if  panelhandle > 0
      % Panel is already open.
      figure( panelhandle );
%      helpdlg( 'The numerical IVP solver tool is already open.', 'Numerical tool is open' );
      return;
   end

   % Start up control panel
   panelhandle = feval( codename, ud );  % e.g., h = w_numivp( ud );
   set( panelhandle, 'HandleVisibility', 'CallBack' );
   pud = get( panelhandle, 'Userdata' );

   set( pud.Handle.NewFig_radio, 'Value', 1 );
   set( pud.Handle.CurrentFig_radio, 'Value', 0 );
   pud.Handle.targetfig = -1;

   switch action
   case { 'numivp' 'visdemo' }
      value = getvalue( lower( char( ud.method ) ) );
      set( pud.Handle.MethodPopUp, 'Value', value );

      if value <= 3  %fixed-step methods
         visible = 'on';
      else
         visible = 'off';
      end

      set( pud.Handle.Choose_text, 'Visible', visible );
      set( pud.Handle.StepSize_radio, 'Visible', visible );
      set( pud.Handle.NumStep_radio, 'Visible', visible );
      set( pud.Handle.StepSize_box, 'Visible', visible );
      set( pud.Handle.NumStep_box, 'Visible', visible );

      set( pud.Handle.StepSize_radio, 'Value', 1 );
      set( pud.Handle.NumStep_radio, 'Value', 0 );
      set( pud.Handle.NumStep_box, 'Enable', 'off' );
      set( pud.Handle.NumStep_box, 'BackgroundColor', [ 0.8 0.8 0.8 ] );
      pud.NumStep = NStep( ud.t0, pud.tf, pud.deltat );

      set( pud.Handle.tf_box, 'String', char( pud.tf ) );

      switch action
      case 'numivp'
         set( pud.Handle.odeplot_radio, 'Value', 1 );
         pud.odeoptions = 'odeplot';
         if ud.numeqn == 2  | ud.numeqn == 3
            set( pud.Handle.odephase_radio, 'Value', 0 );
         end
         set( pud.Handle.nograph_radio, 'Value', 0 );
         set( pud.Handle.workspacearray_check, 'Value', 0 );

         ud.Handle.numivp = panelhandle;

      case 'visdemo'
        set( pud.Handle.lines, 'Checked', 'on' );
        set( pud.Handle.arrows, 'Checked', 'off' );
        pud.style = 'lines';

        set( pud.Handle.UseEnteredIC_radio, 'Value', 1 );
        set( pud.Handle.ClickIC_radio, 'Value', 0 );

        ud.Handle.visdemo = panelhandle;

      end  % switch

   case 'phaseline'
      ud.Handle.phaseline = panelhandle;

   case { 'dfield' 'plotvst' 'phaseplane' 'phasespace' }
      set( pud.Handle.ode23, 'Checked', 'off' );
      set( pud.Handle.ode45, 'Checked', 'on' );
      pud.method = 'ode45';
      set( pud.Handle.ode113, 'Checked', 'off' );
      set( pud.Handle.ode23s, 'Checked', 'off' );
      set( pud.Handle.ode15s, 'Checked', 'off' );

      switch action
      case 'dfield'
         set( pud.Handle.show_phaseline, 'Checked', 'on' );
         set( pud.Handle.hide_phaseline, 'Checked', 'off' );

         pud.numpts = 20;

         set( pud.Handle.lines, 'Checked', 'on' );
         set( pud.Handle.arrows, 'Checked', 'off' );
         pud.style = 'lines';

         set( pud.Handle.UseEnteredIC_radio, 'Value', 1 );
         set( pud.Handle.ClickIC_radio, 'Value', 0 );

         set( pud.Handle.tf_box, 'String', char( pud.tf ) );

         ud.Handle.dfield = panelhandle;

      case 'plotvst'
         if ud.numeqn > 1
            set( pud.Handle.showvar, 'Value', 1 );
         end  %if

         ud.Handle.plotvst = panelhandle;

      case 'phaseplane'
         if ud.numeqn == 2
            set( pud.Handle.show_nullclines, 'Checked', 'on' );
            set( pud.Handle.hide_nullclines, 'Checked', 'off' );

            set( pud.Handle.lines, 'Checked', 'off' );
            set( pud.Handle.arrows, 'Checked', 'on' );
            pud.style = 'arrows';

            set( pud.Handle.velocity, 'Checked', 'off' );
            set( pud.Handle.uniform, 'Checked', 'on' );
            pud.size = 'uniform';

            set( pud.Handle.UseEnteredIC_radio, 'Value', 1 );
            set( pud.Handle.ClickIC_radio, 'Value', 0 );
         else
            set( pud.Handle.showvar, 'Value', 0 );
            set( pud.Handle.showvar(1), 'Value', 1 );
            set( pud.Handle.showvar(2), 'Value', 1 );
         end  % if

         set( pud.Handle.forward, 'Checked', 'on' );
         set( pud.Handle.backward, 'Checked', 'off' );
         set( pud.Handle.both, 'Checked', 'off' );

         ud.Handle.phaseplane = panelhandle;

      case 'phasespace'
         if ud.numeqn > 3
            set( pud.Handle.showvar, 'Value', 0 );
            set( pud.Handle.showvar(1), 'Value', 1 );
            set( pud.Handle.showvar(2), 'Value', 1 );
            set( pud.Handle.showvar(3), 'Value', 1 );
         end  % if

         set( pud.Handle.forward, 'Checked', 'on' );
         set( pud.Handle.backward, 'Checked', 'off' );
         set( pud.Handle.both, 'Checked', 'off' );

         ud.Handle.phasespace = panelhandle;

      end  % switch

   end  % switch

%   setfield( ud.Handle, fieldname, panelhandle );
   set( hin, 'UserData', ud );
   set( panelhandle, 'UserData', pud );

case 'StepSize_radio'
   pud = get( gcbf, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserDat' );

   set( pud.Handle.StepSize_radio, 'Value', 1 );
   set( pud.Handle.NumStep_radio, 'Value', 0 );
   set( pud.Handle.NumStep_box, 'Enable', 'off' );
   set( pud.Handle.NumStep_box, 'BackgroundColor', [ 0.8 0.8 0.8 ] );
   set( pud.Handle.StepSize_box, 'Enable', 'on' );
   set( pud.Handle.StepSize_box, 'BackgroundColor', [ 1 1 1 ] );

   pud.NumStep = NStep( ud.t0, pud.tf, pud.deltat );

   set( gcbf, 'UserData', pud );
%   UpdateOpenPanels ( ud );

case 'NumStep_radio'
   pud = get( gcbf, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserDat' );

   set( pud.Handle.StepSize_radio, 'Value', 0 );
   set( pud.Handle.NumStep_radio, 'Value', 1 );
   set( pud.Handle.NumStep_box, 'Enable', 'on' );
   set( pud.Handle.NumStep_box, 'BackgroundColor', [ 1 1 1 ] );
   set( pud.Handle.StepSize_box, 'Enable', 'off' );
   set( pud.Handle.StepSize_box, 'BackgroundColor', [ 0.8 0.8 0.8 ] );

   pud.deltat = DStep( ud.t0, pud.tf, pud.NumStep );
   set( gcbf, 'UserData', pud );
%   UpdateOpenPanels ( ud );

case 'odeplot'
   pud = get( gcbf, 'Userdata' );

   set( pud.Handle.odeplot_radio, 'Value', 1 );
   pud.odeoptions = 'odeplot';

   if pud.Handle.odephase_radio > 0
      set( pud.Handle.odephase_radio, 'Value', 0 );
   end;

   set( pud.Handle.nograph_radio, 'Value', 0 );

   set( gcbf, 'UserData', pud );
%   UpdateOpenPanels ( ud );

case 'odephas2'
   pud = get( gcbf, 'Userdata' );

   set( pud.Handle.odeplot_radio, 'Value', 0 );
   set( pud.Handle.odephase_radio, 'Value', 1 );
   pud.odeoptions = 'odephas2';
   set( pud.Handle.nograph_radio, 'Value', 0 );

   set( gcbf, 'UserData', pud );
%   UpdateOpenPanels ( ud );

case 'odephas3'
   pud = get( gcbf, 'Userdata' );

   set( pud.Handle.odeplot_radio, 'Value', 0 );
   set( pud.Handle.odephase_radio, 'Value', 1 );
   pud.odeoptions = 'odephas3';
   set( pud.Handle.nograph_radio, 'Value', 0 );

   set( gcbf, 'UserData', pud );
%   UpdateOpenPanels ( ud );

case 'nograph'
   pud = get( gcbf, 'Userdata' );

   set( pud.Handle.odeplot_radio, 'Value', 0 );

   if pud.Handle.odephase_radio > 0
      set( pud.Handle.odephase_radio, 'Value', 0 );
   end;

   set( pud.Handle.nograph_radio, 'Value', 1 );
   pud.odeoptions = '';

   set( gcbf, 'UserData', pud );
%   UpdateOpenPanels ( ud );

case 'UseEnteredIC_radio'
   pud = get( gcbf, 'Userdata' );
   set( pud.Handle.UseEnteredIC_radio, 'Value', 1 );
   set( pud.Handle.ClickIC_radio, 'Value', 0 );

   set( gcbf, 'UserData', pud );
%   UpdateOpenPanels ( ud );

case 'ClickIC_radio'
   pud = get( gcbf, 'Userdata' );
   set( pud.Handle.UseEnteredIC_radio, 'Value', 0 );
   set( pud.Handle.ClickIC_radio, 'Value', 1 );
   set( gcbf, 'UserData', pud );
%   UpdateOpenPanels ( ud );

case 'NewFig_radio'
   pud = get( gcbf, 'Userdata' );
   set( pud.Handle.NewFig_radio, 'Value', 1 );
   set( pud.Handle.CurrentFig_radio, 'Value', 0 );
   pud.Handle.targetfig = -1;
   set( gcbf, 'UserData', pud );
%   UpdateOpenPanels ( ud );

case 'CurrentFig_radio'
   pud = get( gcbf, 'Userdata' );
   set( pud.Handle.NewFig_radio, 'Value', 0 );
   set( pud.Handle.CurrentFig_radio, 'Value', 1 );

   set( gcbf, 'UserData', pud );

case 'targetfig_edit'
   pud = get( gcbf, 'Userdata' );
   pud.Handle.targetfig = ...
      str2num( deblank( get( pud.Handle.figure_box, 'String' ) ) );
   figure( pud.Handle.targetfig );

   % Assume user wants to plot on existing figure if edits this value.
   set( pud.Handle.NewFig_radio, 'Value', 0 );
   set( pud.Handle.CurrentFig_radio, 'Value', 1 );

   set( gcbf, 'UserData', pud );

case 'SetMethod'
   pud = get( gcbf, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserData' );

   if ud.Handle.plotvst == gcbf ...
      | ud.Handle.phaseplane == gcbf ...
      | ud.Handle.phasespace == gcbf ...
      | ud.Handle.dfield == gcbf
      % Called by plotvst, phaseplane, phasespace, or dfield, which have
      % method option in menu bar
      menubar = 1;
      set( pud.Handle.ode23, 'Checked', 'off' );
      set( pud.Handle.ode45, 'Checked', 'off' );
      set( pud.Handle.ode113, 'Checked', 'off' );
      set( pud.Handle.ode23s, 'Checked', 'off' );
      set( pud.Handle.ode15s, 'Checked', 'off' );
   else
      menubar = 0;
   end

   switch inputnumber
   case 1
      pud.method = 'Euler';
      visible = 'on';

   case 2
      pud.method = 'Heun';
      visible = 'on';

   case 3
      pud.method = 'RK4';
      visible = 'on';

   case 4
      pud.method = 'ode23';
      visible = 'off';
      if menubar
         set( pud.Handle.ode23, 'Checked', 'on' );
      end

   case 5
      pud.method = 'ode45';
      visible = 'off';
      if menubar
         set( pud.Handle.ode45, 'Checked', 'on' );
      end

   case 6
      pud.method = 'ode113';
      visible = 'off';
      if menubar
         set( pud.Handle.ode113, 'Checked', 'on' );
      end

   case 7
      pud.method = 'ode23s';
      visible = 'off';
      if menubar
         set( pud.Handle.ode23s, 'Checked', 'on' );
      end

   case 8
      pud.method = 'ode15s';
      visible = 'off';
      if menubar
         set( pud.Handle.ode15s, 'Checked', 'on' );
      end

   end  % switch inputnumber

   if  ud.Handle.numivp == gcbf  |  ud.Handle.visdemo == gcbf
      % Called by NumIVP or visdemo
      set( pud.Handle.Choose_text, 'Visible', visible );
      set( pud.Handle.StepSize_radio, 'Visible', visible );
      set( pud.Handle.NumStep_radio, 'Visible', visible );
      set( pud.Handle.StepSize_box, 'Visible', visible );
      set( pud.Handle.NumStep_box, 'Visible', visible );
   end

   set( gcbf, 'UserData', pud );
%   UpdateOpenPanels ( ud );

case 'StartNumIVP'
   pud = get( gcbf, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserData' );
   set( gcbf, 'Pointer', 'watch' );

   % inline function to feed ode solver
   g = makeinline( ud );

%   % Replace parameters with their values.
   y0cell = prm2val( ud.y0, ud.pname, ud.pvalue );

   t0 = str2num( char( ud.t0 ) );
   tf = str2num( char( pud.tf ) );
   y0 = str2num( char( y0cell ) );;

   try
      feval( g, t0, y0 );
   catch
      set( gcbf, 'Pointer', 'arrow' );
      errordlg( ...
      'Faulty problem specification.  All parameters need numeric values.',...
         'Error' );
      return;
   end

   % Set up figure, if needed.
   if  get( pud.Handle.nograph_radio, 'Value' ) == 0

      if  get( pud.Handle.NewFig_radio, 'Value' ) == 1 ...
         | pud.Handle.targetfig == -1
         % new figure
         pud.Handle.targetfig = figure;
         plotedit on;
      else
         figure( pud.Handle.targetfig );
         hold on;
      end

   end

   % Solve IVP.
   % Shouldn't need all these cases - prob with feval (or use
   % OutputFnc option)
   optstring = pud.odeoptions;
   options = odeset( 'OutputFcn', optstring );
   switch lower( pud.method )
   case 'euler'
      h = str2num( pud.deltat );
      [tout yout] = ode_euler( g, [t0 tf], y0, h, options );
   case 'heun'
      h = str2num( pud.deltat );
      [tout yout] = ode_heun( g, [t0 tf], y0, h, options );
   case 'rk4'
      h = str2num( pud.deltat );
      [tout yout] = ode_rk4( g, [t0 tf], y0, h, options );
   case 'ode23'
      [tout yout] = ode23( g, [t0 tf], y0, options );
   case 'ode45'
      [tout yout] = ode45( g, [t0 tf], y0, options );
   case 'ode113'
      [tout yout] = ode113( g, [t0 tf], y0, options );
   case 'ode23s'
      [tout yout] = ode23s( g, [t0 tf], y0, options );
   case 'ode15s'
      [tout yout] = ode15s( g, [t0 tf], y0, options );
   end % switch

   if ~strcmp( optstring, '' )
      % Some graphical output requested
      titlestring = [ 'Via ' char( pud.method ) ': ' char( ud.y{1} ) ''' = ' char( ud.f{1} ) ];

      for i = 2:ud.numeqn
         titlestring = [ titlestring ', ' char( ud.y{i} ) ''' = ' char( ud.f{i} ) ];
      end

      for i = 1:length( ud.pvalue )
         if ~isempty( ud.pvalue{i} )
            titlestring = [ titlestring ', ' char( ud.pname{i} ) ' = ' char( ud.pvalue{i} ) ];
         end
      end

      feval( 'title', titlestring );

      switch lower( optstring )
      case 'odeplot'
         feval( 'xlabel', char( ud.t ) );
         if ud.numeqn > 1
            legendtext = char( ud.y );
            legend( legendtext );
         else
            feval( 'ylabel', char( ud.y{1} ) );
         end;

      case 'odephas2'
         feval( 'xlabel', char( ud.y{1} ) );
         feval( 'ylabel', char( ud.y{2} ) );

      case 'odephas3'
         feval( 'xlabel', char( ud.y{1} ) );
         feval( 'ylabel', char( ud.y{2} ) );
         feval( 'zlabel', char( ud.y{3} ) );

      end  % switch

      %targetfig = gcf;
      pud.Handle.targetfig = gcf;
      hold off;

   end

   % Store array in workspace.
   if get( pud.Handle.workspacearray_check, 'Value' ) == 1
      [ solname, number ] = UniqueName( ud.y, gcf );
      toutname = [ char( ud.t ) num2str( number ) ];
      tname = char( ud.t );
      assignin( 'base', toutname, tout );
      assignin( 'base', solname, yout );
      msgbox( [ 'The approximate solution is available in MATLAB in the ',...
         'arrays ' toutname ' (' tname ' values) and ' solname,...
         ' (approximate solution values).' ], ...
         'Numeric solution in workspace' );
   end

   set( gcbf, 'Pointer', 'arrow' );

   set( gcbf, 'UserData', pud );
   set( pud.Handle.order1, 'UserData', ud );

case 'Startphaseline'
   pud = get( gcbf, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserData' );
   set( gcbf, 'Pointer', 'watch' );

   % Autonomous?
   fstring = prm2val( ud.f, ud.pname, ud.pvalue, ud.y );
   if char( diff( fstring{1}, ud.t ) ) ~= '0'
      set( gcbf, 'Pointer', 'arrow' );
      errordlg( ['This equation is not autonomous; no phase line is available.' ],'Error' );
      return;
   end

   % Construct inline function.
   fstring = char( prm2val( ud.f, ud.pname, ud.pvalue, ud.y ) );
   tname = char( ud.t );
   yname = char( ud.y{1} );
   func = vectorize( inline( fstring, yname ) );

   ymin = str2num( char( pud.ymin ) );
   ymax = str2num( char( pud.ymax ) );
   dely = (ymax - ymin)/127;
   yvalues = ymin:dely:ymax;

   try
      fvalues = func( yvalues );
   catch
      errordlg( 'Faulty problem formulation.  Have parameters been assigned numeric values?', 'Error' );
      return;
   end;

   try
      lastwarn = '';
      roots = double( feval( 'solve', [ fstring '= 0'] ) );

      if ~isempty( lastwarn )
         warndlg( lastwarn, 'Warning' );
         lastwarn( '' );
      end

   catch

      if ~isempty( lastwarn )
         warndlg( lastwarn, 'Warning' );
         lastwarn( '' );
      elseif ~isempty( lasterr )
         errordlg( lasterr, 'Error' );
      else
         warndlg( 'No roots of right-hand side or faulty problem formulation', 'Warning' );
      end  % if

      return;

   end  % try

   % Plot
   if get( pud.Handle.NewFig_radio, 'Value' ) == 1 ...
      | pud.Handle.targetfig == -1
      % Start a new figure
      pud.Handle.targetfig = figure;
      plotedit on;
      hold on;
   else
      % Use previous figure.
      figure( pud.Handle.targetfig );
      hold on;
   end;

%   co = get( gca, 'ColorOrder' );
   black = [ 0 0 0 ];
   gray = [ .5 .5 .5 ];
   plot( yvalues, fvalues ); %, 'Color', co(1, :) );
   plot( [ ymin; ymax ], [ 0; 0 ], '-', 'Color', black,...
      'LineWidth', 2 );
   a = axis; a = a';
   delf = (a(4) - a(3))/20;

%   size = 2*get( gca, 'MarkerSize' );
   size = 12;
   edgecolor = black;
   for i = 1:length( roots )

      if imag( roots(i) ) == 0 ...
         & roots(i) >= ymin ...
         & roots(i) <= ymax
         left = func( roots(i) - dely );
         right = func( roots(i) + dely );

         if left > 0 & right < 0
            % stable
            shape = 'o';
            facecolor = black;
            string = 'Stable';
         elseif left < 0 & right > 0
            % unstable
            shape = 'o';
            facecolor = 'auto';
            string = 'Unstable';
         else
            shape = 'o';
            facecolor = gray;
            string = '';
         end  % if

         plot( roots(i), 0, 'Marker', shape, 'Color', black, ...
            'MarkerEdgeColor', edgecolor, ...
            'MarkerFaceColor', facecolor, ...
            'MarkerSize', size );
         text( roots(i), delf, string, 'HorizontalAlignment', 'center' );
      end  %if

   end  % for

   % Title and axis labels
   ystring = [ char( ud.y{1} ) ''' = ' char( ud.f{1} ) ];

   for i = 1:length( ud.pvalue )
      if ~isempty( ud.pvalue{i} )
         ystring = [ ystring ', ' char( ud.pname{i} ) ' = ' char( ud.pvalue{i} ) ];
      end
   end

   feval( 'ylabel', ystring );
   feval( 'xlabel', yname );

   s = prettytitle( char( ud.f ) );
   feval( 'title', [ 'Phase line: d' yname '/d' tname ' = ' s ] );
   hold off;

   set( gcbf, 'Pointer', 'arrow' );

   % Update "parent" panel.
   set( ud.Handle.phaseline, 'UserData', pud );

case 'Startdfield'
   pud = get( gcbf, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserData' );
   set( gcbf, 'Pointer', 'watch' );

   % Construct inline function and ode call arguments
   fstring = char( prm2val( ud.f, ud.pname, ud.pvalue, ud.y ) );
   tname = char( ud.t );
   yname = char( ud.y{1} );
   yp = vectorize( inline( fstring, tname, yname ) );

   y0cell = prm2val( ud.y0, ud.pname, ud.pvalue );
   t0 = str2num( char( ud.t0 ) );
   tf = str2num( char( pud.tf ) );
   y0 = str2num( char( y0cell ) );

%   y0string = char( prm2val( ud.y0, ud.pname, ud.pvalue ) );
   %y0string = char( ud.y0 );

   % Test of yp
   try
      feval( yp, t0, y0 );
   catch
      set( gcbf, 'Pointer', 'arrow' );
      errordlg( 'Faulty problem formulation.  Have parameters and initial conditions been assigned numeric values?', 'Error' );
      return;
   end;

   % Figure set up
   if get( pud.Handle.NewFig_radio, 'Value' ) == 1 ...
      | pud.Handle.targetfig == -1
      % Start a new figure
      ymin = str2num( char( pud.ymin ) );
      ymax = str2num( char( pud.ymax ) );
      a = [ t0 tf ymin ymax ];
      pud.Handle.targetfig = figure;
      plotedit on;
      axis( a );
      axis manual;
      hold on;
      dfld1( pud.numpts, a, yp, pud.style );
   else
      % Use previous figure.
      figure( pud.Handle.targetfig );
      axis manual;
      hold on;
   end;

   % Phase line, if requested
   s = get( pud.Handle.show_phaseline, 'Checked' );
   if strcmp( s, 'on' )  % Draw (vertical) phase line
      ymin = str2num( char( pud.ymin ) );
      ymax = str2num( char( pud.ymax ) );
      dely = (ymax - ymin)/127;
      yvalues = ymin:dely:ymax;

      try
         lastwarn = '';
         roots = double( feval( 'solve', [ fstring '= 0'] ) );

         if ~isempty( lastwarn )
            warndlg( lastwarn, 'Warning' );
            lastwarn( '' );
         end

      catch

         if ~isempty( lastwarn )
            warndlg( lastwarn, 'Warning' );
            lastwarn( '' );
         elseif ~isempty( lasterr )
            errordlg( lasterr, 'Error' );
         else
            warndlg( 'No roots of right-hand side or faulty problem formulation', 'Warning' );
         end  % if

         set( gcbf, 'Pointer', 'arrow' );
         return;

      end  % try

      % Plot
      black = [ 0 0 0 ];
      gray = [ .5 .5 .5 ];
      plot( [ t0; t0 ], [ ymin; ymax ], '-', 'Color', black,...
         'LineWidth', 4 );
      delt = (tf - t0)/50;

      size = 8;
      edgecolor = black;
      vtol = 3*dely;
      for i = 1:length( roots )

         if imag( roots(i) ) == 0 ...
            & roots(i) >= ymin ...
            & roots(i) <= ymax
            left = yp( t0, roots(i) - dely );
            right = yp( t0, roots(i) + dely );

            if left > 0 & right < 0
               % stable
               shape = 'o';
               facecolor = black;
               string = 'Stable';
            elseif left < 0 & right > 0
               % unstable
               shape = 'o';
               facecolor = 'auto';
               string = 'Unstable';
            else
               shape = 'o';
               facecolor = gray;
               string = '';
            end  % if

            plot( t0, roots(i), 'Marker', shape, 'Color', black, ...
               'MarkerEdgeColor', edgecolor, ...
               'MarkerFaceColor', facecolor, ...
               'MarkerSize', size );

            if abs( roots(i) - ymin ) < vtol
               ycoord = roots(i) + vtol;
            else
               ycoord = roots(i);
            end  % if

            text( delt, ycoord, string, 'HorizontalAlignment', 'left' );
         end  %if

      end  % for

   end  % if


   % Solution plot title
   titlestring = [ char( ud.y{1} ) ''' = ' char( ud.f{1} ) ];

   for i = 1:length( ud.pvalue )
      if ~isempty( ud.pvalue{i} )
         titlestring = [ titlestring ', ' char( ud.pname{i} ) ' = ' char( ud.pvalue{i} ) ];
      end
   end

   feval( 'title', titlestring );
   %s = prettytitle( fstring );
   %title( [ 'd' yname '/d' tname ' = ' s ] );
   feval( 'xlabel', tname );
   feval( 'ylabel', yname );

   % Initial conditions if click on figure
   if get( pud.Handle.UseEnteredIC_radio, 'Value' ) ~= 1
      % Click on figure
      set( gcbf, 'Pointer', 'arrow' );
      set( pud.Handle.ClickText, 'Visible', 'on' );
      [t0 y0 ] = ginput(1);
      figure( pud.Handle.targetfig );
      set( pud.Handle.ClickText, 'Visible', 'off' );
      set( gcbf, 'Pointer', 'watch' );
   end;

   options = odeset( 'OutputFcn', 'delab_odeplot' );
   feval( pud.method, yp, [t0 tf], y0, options );
   set( gcbf, 'Pointer', 'arrow' );

   % Update "parent" equation windows.
   set( ud.Handle.dfield, 'UserData', pud );

case 'Startvisdemo'
   pud = get( gcbf, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserData' );
   SolvDraw( ud, pud );
%   set( gcbf, 'UserData', ud );

case 'Startplotvst'
   pud = get( gcbf, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserData' );
   set( gcbf, 'Pointer', 'watch' );

   % inline function to feed ode solver
   g = makeinline( ud );

   % Replace parameters with their values.
   y0cell = prm2val( ud.y0, ud.pname, ud.pvalue );

   t0 = str2num( char( ud.t0 ) );
   tf = str2num( char( pud.tf ) );
   y0 = str2num( char( y0cell ) );

   try
      feval( g, t0, y0 );
   catch
      set( gcbf, 'Pointer', 'arrow' );
      errordlg( ...
      'Faulty problem specification.  All parameters need numeric values.',...
         'Error' );
      return;
   end

   % Output variables.
   if ud.numeqn > 1
      showvar = get( pud.Handle.showvar, 'value' );
   else
      showvar = [ ];
   end  % if
   [ outlist, ylist, outcount ] = vartoshow( ud.numeqn, ud.y, showvar );

   if outcount == 0
      set( gcbf, 'Pointer', 'arrow' );
      errordlg( 'No variables selected to show on graph', 'Error' );
      return;
   end  % if

   % Set up figure
   if  get( pud.Handle.NewFig_radio, 'Value' ) == 1 ...
      | pud.Handle.targetfig == -1
      % new figure
      pud.Handle.targetfig = figure;
      plotedit on;
   else
      figure( pud.Handle.targetfig );
      hold on;
   end

   % Solve IVP
   options = odeset( 'OutputFcn', 'delab_odeplot', ...
      'OutputSel', outlist );
   feval( pud.method, g, [t0 tf], y0, options );

   titlestring = [ char( ud.y{1} ) ''' = ' char( ud.f{1} ) ];
   for i = 2:ud.numeqn
      titlestring = [ titlestring ', ' char( ud.y{i} ) ''' = ' char( ud.f{i} ) ];
   end
   for i = 1:length( ud.pvalue )
      if ~isempty( ud.pvalue{i} )
         titlestring = [ titlestring ', ' char( ud.pname{i} ) ' = ' char( ud.pvalue{i} ) ];
      end
   end
   feval( 'title', titlestring );

   feval( 'xlabel', char( ud.t ) );
   if outcount > 1
      legendtext = char( ylist );
      feval( 'legend', legendtext );
   else
      feval( 'ylabel', char( ud.y{ outlist(1) } ) );
   end;

   set( gcbf, 'Pointer', 'arrow' );

   pud.Handle.targetfig = gcf;
   hold off;
   set( gcbf, 'UserData', pud );

case 'SolveForward'
   pud = get( gcbf, 'UserData' );

%   if strcmp( get( pud.Handle.forward, 'Checked' ), 'off' )
      set( pud.Handle.forward, 'Checked', 'on' );
      set( pud.Handle.backward, 'Checked', 'off' );
      set( pud.Handle.both, 'Checked', 'off' );
%   else
%      set( pud.Handle.forward, 'Checked', 'off' );
%   end

   set( gcbf, 'UserData', pud );

case 'SolveBackward'
   pud = get( gcbf, 'UserData' );

%   if strcmp( get( pud.Handle.backward, 'Checked' ), 'off' )
      set( pud.Handle.backward, 'Checked', 'on' );
      set( pud.Handle.forward, 'Checked', 'off' );
      set( pud.Handle.both, 'Checked', 'off' );
%   else
%      set( pud.Handle.backward, 'Checked', 'off' );
%   end

   set( gcbf, 'UserData', pud );

case 'SolveForandBack'
   pud = get( gcbf, 'UserData' );

   set( pud.Handle.forward, 'Checked', 'off' );
   set( pud.Handle.backward, 'Checked', 'off' );
   set( pud.Handle.both, 'Checked', 'on' );

   set( gcbf, 'UserData', pud );

case 'numpts'

   if inputnumber > 0
      hcallback = inputnumber;  % f_order1 has called itself
   else
      hcallback = gcbf;
   end;

   pud = get( hcallback, 'UserData' );
   ud = get( pud.Handle.order1, 'UserData' );
   prompt = [ 'Enter number of horizontal and vertical flow lines.' ];
   title = [ 'Number of flow lines' ];
   def = { num2str( pud.numpts ) };
   answer = inputdlg( prompt, title, [ 1 ( 7 + length( prompt ) ) ], def );

   if ~isempty( answer )
      numpts = str2num( answer{1} );
   else
      return;
   end

   if ~isnumeric( numpts ) | isempty( numpts )
      h = errordlg( 'Enter a numeric value.', 'Error' );
      waitfor( h );
      f_order1 numpts hcallback;
      return;
   elseif numpts < 0 | numpts > 100
      h = errordlg( 'Enter a nonnegative value no greater than 100.', 'Error' );
      waitfor( h );
      f_order1 numpts hcallback;
      return;
   end

   pud.numpts = ceil( numpts );

   set( hcallback, 'UserData', pud );

case 'tfdialog'

   if inputnumber > 0
      hcallback = inputnumber;  % f_order1 has called itself
   else
      hcallback = gcbf;
   end;

   pud = get( hcallback, 'UserData' );
   ud = get( pud.Handle.order1, 'UserData' );
   prompt = [ 'Enter last ' char( ud.t ) '; choose a value greater than ' char( ud.t0 ) '.' ];
   title = [ 'Enter last ' char( ud.t ) ];
   def = { num2str( pud.tf ) };
   answer = inputdlg( prompt, title, [ 1 (7 + length( prompt ) )], def );

   if ~isempty( answer )
      tf = str2num( answer{1} );
   else
      return;  % cancelled
   end

   t0 = str2num( ud.t0 );
%   c = le( tf, t0 )  % returns c = 1 with t0 = 0, tf = 6 ???
%   if ~isnumeric( tf ) | isempty( tf )
%      h = errordlg( 'Enter a numeric value.', 'Error' );
%      waitfor( h );
%      f_order1 tfdialog hcallback;
%      return;
%   elseif  tf <= t0
   if tf <= t0
      h = errordlg( [ 'Enter a value greater than ' char( ud.t0 ) '.' ], 'Error' );
      waitfor( h );
      f_order1 tfdialog hcallback;
      return;
   end

   pud.tf = answer{1};

   set( hcallback, 'UserData', pud );

case 'show_phaseline'
   pud = get( gcbf, 'UserData' );

   set( pud.Handle.show_phaseline, 'Checked', 'on' );
   set( pud.Handle.hide_phaseline, 'Checked', 'off' );
   set( gcbf, 'UserData', pud );

case 'hide_phaseline'
   pud = get( gcbf, 'UserData' );

   set( pud.Handle.show_phaseline, 'Checked', 'off' );
   set( pud.Handle.hide_phaseline, 'Checked', 'on' );
   set( gcbf, 'UserData', pud );

case 'uniform'
   pud = get( gcbf, 'UserData' );

   set( pud.Handle.velocity, 'Checked', 'off' );
   set( pud.Handle.uniform, 'Checked', 'on' );
   pud.size = 'uniform';

   set( gcbf, 'UserData', pud );

case 'velocity'
   pud = get( gcbf, 'UserData' );

   set( pud.Handle.velocity, 'Checked', 'on' );
   set( pud.Handle.uniform, 'Checked', 'off' );
   pud.size = 'velocity';

   set( gcbf, 'UserData', pud );

case 'arrows'
   pud = get( gcbf, 'UserData' );

   set( pud.Handle.lines, 'Checked', 'off' );
   set( pud.Handle.arrows, 'Checked', 'on' );
   pud.style = 'arrows';

   set( gcbf, 'UserData', pud );

case 'lines'
   pud = get( gcbf, 'UserData' );

   set( pud.Handle.lines, 'Checked', 'on' );
   set( pud.Handle.arrows, 'Checked', 'off' );
   pud.style = 'lines';

   set( gcbf, 'UserData', pud );

case 'show_nullclines'
   pud = get( gcbf, 'UserData' );

   set( pud.Handle.show_nullclines, 'Checked', 'on' );
   set( pud.Handle.hide_nullclines, 'Checked', 'off' );
   set( gcbf, 'UserData', pud );

case 'hide_nullclines'
   pud = get( gcbf, 'UserData' );

   set( pud.Handle.show_nullclines, 'Checked', 'off' );
   set( pud.Handle.hide_nullclines, 'Checked', 'on' );
   set( gcbf, 'UserData', pud );

case 'Startphaseplane'
   pud = get( gcbf, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserData' );
   set( gcbf, 'Pointer', 'watch' );

   % Output variables.
   if ud.numeqn > 2
      showvar = get( pud.Handle.showvar, 'value' );
      [ outlist, ylist, outcount ] = vartoshow( ud.numeqn, ud.y, showvar );

      if outcount ~= 2
         set( gcbf, 'Pointer', 'arrow' );
         errordlg( 'Choose exactly TWO variables to show on phase plane.', 'Error' );
         return;
      end  % if
   else
      outlist = [ 1; 2 ];
      ylist = ud.y;
   end  % if

   if  get( pud.Handle.NewFig_radio, 'Value' ) == 1 ...
      | pud.Handle.targetfig == -1
      % new figure
      pud.Handle.targetfig = figure;
      plotedit on;

      if ud.numeqn ==2
         axis manual;
         axis( [ str2num( pud.ymin{1} )  str2num( pud.ymax{1} ) ...
            str2num( pud.ymin{2} )  str2num( pud.ymax{2} ) ] );
         hold on;
      end  % if

      titlestring = [ char( ud.y{1} ) ''' = ' char( ud.f{1} ) ];
      for i = 2:ud.numeqn
         titlestring = [ titlestring ', ' char( ud.y{i} ) ''' = ' char( ud.f{i} ) ];
      end
      for i = 1:length( ud.pvalue )
         if ~isempty( ud.pvalue{i} )
            titlestring = [ titlestring ', ' char( ud.pname{i} ) ' = ' char( ud.pvalue{i} ) ];
         end
      end
      feval( 'title', titlestring );

      feval( 'xlabel', char( ylist{1} ) );
      feval( 'ylabel', char( ylist{2} ) );
   else
      figure( pud.Handle.targetfig );
      hold on;
   end

   % inline function to feed ode solver
   g = makeinline( ud );

   if  ud.numeqn == 2 & ...
      get( pud.Handle.NewFig_radio, 'Value' ) == 1 ...
      | pud.Handle.targetfig == -1
      % New figure for 2 equations; draw direction field
      a = [ str2num( pud.ymin{1} )  str2num( pud.ymax{1} ) ...
            str2num( pud.ymin{2} )  str2num( pud.ymax{2} ) ];
      if pud.numpts > 0
         dfld2( pud.numpts, a, g, str2num( ud.t0 ), pud.style, pud.size );
      end  % if

      s = get( pud.Handle.show_nullclines, 'Checked' );
      if strcmp( s, 'on' )  % Draw nullclines
         f = prm2val( ud.f, ud.pname, ud.pvalue );
         colorlist = get( gca, 'ColorOrder' );
         lcl = length( colorlist );
         legendtext = cell(2, 1);
         legendh = zeros(2, 1);

         numpts = 128;
         yrange = zeros(numpts, 2);
         for i = 1:2
            ymin = str2num( pud.ymin{i} );
            ymax = str2num( pud.ymax{i} );
            dely = (ymax - ymin)/(numpts - 1);
            yrange(:,i) = [ ymin:dely:ymax ]';
         end  % for

         warning off;
         for i = 1:2
            % Solve i-th rhs = 0 for each unknown, then plot
            % the resulting nullcline.  (Overlap is possible.)
%            fprintf( 'Solving equation %i...\n', i );
            h = 0;

            try
%               fprintf( '... for y1 in terms of y2\n' );
               y1 = solve( [ f{i} '= 0' ], ud.y{1} );

               for j = 1:length( y1 )
                  y1val = double( subs( y1(j), ud.y{2}, yrange(:, 2) ) );
                  y1val = y1val(:);

                  if length( y1val ) == 1
                     % y1val is a constant
                     y1val = y1val*ones(numpts, 1);
                  end  % if

%                  disp( 'About to plot y1, yrange(:, 2)' );
                  if isreal( y1val )
                     h = feval( 'plot', y1val, yrange(:,2), ...
                        'LineStyle', '-', ...
                        'Color', colorlist(lcl - i, :) );
                  end  % if

               end  % for

            catch;
%               disp( 'No y1 values in terms of y2' );
            end

            try
%               fprintf( '... for y2 in terms of y1\n' );
               y2 = solve( [ f{i} '= 0' ], ud.y{2} );

               for j = 1:length( y2 )
                  y2val = double( subs( y2(j), ud.y{1}, yrange(:, 1) ) );
                  y2val = y2val(:);

                  if length( y2val ) == 1
                     % y2 is a constant
                     y2val = y2val*ones(numpts, 1);
                  end  % if

%                  disp( 'About to plot y2, yrange(:, 1)' );
                  if isreal( y2val )
                     h = feval( 'plot', yrange(:,1), y2val, ...
                        'LineStyle', '-', ...
                        'Color', colorlist(lcl - i, :) );
                  end  % if

               end  % for

            catch;
%               disp( 'No y2 values in terms of y1' );
            end

            if h > 0
               legendtext{i} = [ char( ud.y{i} ) ''' = 0' ];
               legendh(i) = h;
            end  % if

         end  % for
         warning on;

         mask = legendh > 0;
         legend( legendh(mask), char( legendtext(mask) ) );
      end  % if

   end

   % Initial conditions
   t0 = str2num( char( ud.t0 ) );
   if ud.numeqn == 2

      if get( pud.Handle.UseEnteredIC_radio, 'Value' ) == 1
         % Replace parameters with their values.
         y0cell = prm2val( ud.y0, ud.pname, ud.pvalue );
         y0 = str2num( char( y0cell ) );
      else
         % Click on figure
         set( gcbf, 'Pointer', 'arrow' );
         set( pud.Handle.ClickText, 'Visible', 'on' );
         y0 = ginput(1);
         set( pud.Handle.ClickText, 'Visible', 'off' );
         set( gcbf, 'Pointer', 'watch' );
      end;

   else
         y0cell = prm2val( ud.y0, ud.pname, ud.pvalue );
         y0 = str2num( char( y0cell ) );
   end  % if

   tf = str2num( pud.tf );

   try
      feval( g, t0, y0 );
   catch
      set( gcbf, 'Pointer', 'arrow' );
      errordlg( ...
      'Faulty problem specification.  All parameters need numeric values.',...
         'Error' );
      return;
   end

   % Solve IVP.
   options = odeset( 'OutputFcn', 'delab_odephas2', ...
      'OutputSel', outlist );

   if strcmp( get( pud.Handle.forward, 'Checked' ), 'on' ) | ...
      strcmp( get( pud.Handle.both, 'Checked' ), 'on' )
      feval( pud.method, g, [t0 tf], y0, options );
   end

   if strcmp( get( pud.Handle.backward, 'Checked' ), 'on' ) | ...
      strcmp( get( pud.Handle.both, 'Checked' ), 'on' )
      figure( pud.Handle.targetfig );
      feval( pud.method, g, [t0 -tf], y0, options );
   end

   if ud.numeqn > 2
      % Hold on doesn't preserve these from earlier.
      feval( 'title', titlestring );
      feval( 'xlabel', char( ylist(1) ) );
      feval( 'ylabel', char( ylist(2) ) );
   end  % if

   set( gcbf, 'Pointer', 'arrow' );

   pud.Handle.targetfig = gcf;
   hold off;
   set( gcbf, 'UserData', pud );

case 'Startphasespace'
   pud = get( gcbf, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserData' );
   set( gcbf, 'Pointer', 'watch' );

   % Output variables.
   if ud.numeqn > 3
      showvar = get( pud.Handle.showvar, 'value' );
      [ outlist, ylist, outcount ] = vartoshow( ud.numeqn, ud.y, showvar );

      if outcount ~= 3
         set( gcbf, 'Pointer', 'arrow' );
         errordlg( 'Choose exactly THREE variables to show in phase space.', 'Error' );
         return;
      end  % if
   else
      outlist = [ 1; 2; 3 ];
      ylist = ud.y;
   end  % if

   if  get( pud.Handle.NewFig_radio, 'Value' ) == 1 ...
      | pud.Handle.targetfig == -1
      % new figure
      pud.Handle.targetfig = figure;
      plotedit on;

      titlestring = [ char( ud.y{1} ) ''' = ' char( ud.f{1} ) ];
      for i = 2:ud.numeqn
         titlestring = [ titlestring ', ' char( ud.y{i} ) ''' = ' char( ud.f{i} ) ];
      end
      for i = 1:length( ud.pvalue )
         if ~isempty( ud.pvalue{i} )
            titlestring = [ titlestring ', ' char( ud.pname{i} ) ' = ' char( ud.pvalue{i} ) ];
         end
      end

   else
      figure( pud.Handle.targetfig );
      hold on;
   end

   % inline function to feed ode solver
   g = makeinline( ud );

   % Initial conditions
   t0 = str2num( char( ud.t0 ) );
   y0cell = prm2val( ud.y0, ud.pname, ud.pvalue );
   y0 = str2num( char( y0cell ) );
   tf = str2num( pud.tf );

   try
      feval( g, t0, y0 );
   catch
      set( gcbf, 'Pointer', 'arrow' );
      errordlg( ...
      'Faulty problem specification.  All parameters need numeric values.',...
         'Error' );
      return;
   end

   % Solve IVP.
   options = odeset( 'OutputFcn', 'delab_odephas3', ...
      'OutputSel', outlist );

   if strcmp( get( pud.Handle.forward, 'Checked' ), 'on' ) | ...
      strcmp( get( pud.Handle.both, 'Checked' ), 'on' )
      feval( pud.method, g, [t0 tf], y0, options );
   end

   if strcmp( get( pud.Handle.backward, 'Checked' ), 'on' ) | ...
      strcmp( get( pud.Handle.both, 'Checked' ), 'on' )
      figure( pud.Handle.targetfig );
      feval( pud.method, g, [t0 -tf], y0, options );
   end

   feval( 'title', titlestring );
   feval( 'xlabel', char( ylist(1) ) );
   feval( 'ylabel', char( ylist(2) ) );
   feval( 'zlabel', char( ylist(3) ) );
   set( gcbf, 'Pointer', 'arrow' );

   pud.Handle.targetfig = gcf;
   hold off;
   set( gcbf, 'UserData', pud );

case 'AddParameter'
   ud = get( gcbf, 'Userdata' );
   ud.nump = ud.nump + 1;
   ud.pname = ud.pname(:);
   ud.pname = [ ud.pname; { '' } ];
   ud.pvalue = ud.pvalue(:);
   ud.pvalue = [ ud.pvalue; { '' } ];
   close( gcbf );
   h = w_order1( ud );
   set( h, 'HandleVisibility', 'CallBack' );

case 'closepanel'
   h = gcbf;
   pud = get( h, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserData' );

   if h == ud.Handle.numivp
      ud.Handle.numivp = -1;
   elseif h == ud.Handle.visdemo
      ud.Handle.visdemo = -1;
   elseif h == ud.Handle.dfield
      ud.Handle.dfield = -1;
   elseif h == ud.Handle.phaseline
      ud.Handle.phaseline = -1;
   elseif h == ud.Handle.plotvst
      ud.Handle.plotvst = -1;
   elseif h == ud.Handle.phaseplane
      ud.Handle.phaseplane = -1;
   elseif h == ud.Handle.phasespace
      ud.Handle.phasespace = -1;
   end;

   set( ud.Handle.order1, 'UserData', ud );
   clear pud;
   close gcbf;  % close the window that called f_order1

case 'close'
   close gcbf;  % close the window that called f_order1

case 'quit'
   % are you sure?
   close all;

% All edit boxes in equation window
case { 'y_edit' 't_edit' 'f_edit' 'pname_edit' ...
   'pvalue_edit' 't0_edit' 'y0_edit'  'all_edit' }

   if inputnumber > 0
      h = inputnumber;
   else
      h = gcbf;
   end;

   ud = get( h, 'Userdata' );

   switch action
   case 'y_edit'
      ud.y = cell( ud.numeqn, 1 );

      for i = 1:ud.numeqn
          ud.y{i} = char( deblank( get( ud.Handle.y_box(i), 'String' ) ));
      end

%      ud.y = deblank( get( ud.Handle.y_box, 'String' ) );

      if ud.numeqn == 1
            set( ud.Handle.yparen_box, 'String', [ char( ud.y ) '( ' ] );
      else
         for i = 1:ud.numeqn
            set( ud.Handle.yparen_box(i), 'String', [ char( ud.y{i} ) '( ' ] );
         end
      end  % if

   case 't_edit'
      ud.t = deblank( get( ud.Handle.t_box(1), 'String' ) );
      set( ud.Handle.t_box, 'String', char( ud.t ) );

   case 'f_edit'
      ud.f = cell( ud.numeqn, 1 );

      for i = 1:ud.numeqn
          ud.f{i} = char( deblank( get( ud.Handle.f_box(i), 'String' ) ));
      end

   case 'pname_edit'
      len = length( ud.pname );
      ud.pname = cell( len, 1 );
      for i = 1:len
          ud.pname{i} = char( deblank( get( ud.Handle.pname_box(i), 'String' ) ));
      end

      set( gcbf, 'Userdata', ud );

   case 'pvalue_edit'
      % check only digits, ., blanks entered
      len = length( ud.pvalue );
      ud.pvalue = cell( len, 1 );

      for i = 1:len
          ud.pvalue{i} = char( deblank( get( ud.Handle.pvalue_box(i), 'String' ) ));
      end

      set( gcbf, 'Userdata', ud );
   %   ud.pvalue = deblank( get( ud.Handle.pvalue_box, 'String' ) );
   %   % Test for added parameters (nonblank parameter name)
   %   nump = ud.nump;
   %   while  length( ud.pvalue ) > nump ....
   %      & ~isempty( ud.pvalue{nump + 1} )
   %      nump = nump + 1;
   %   end
   %   ud.nump = nump;

   case 't0_edit'
      ud.t0 = deblank( get( ud.Handle.t0_box(1), 'String' ) );
      set( ud.Handle.t0_box, 'String', char( ud.t0 ) );

      if ud.Handle.numivp > 0 | ud.Handle.visdemo > 0
         pudh = max( ud.Handle.numivp, ud.Handle.visdemo );
         pud = get( pudh, 'UserData' );
         pud.NumStep = NStep( ud.t0, ud.tf, ud.deltat );
         set( pud.Handle.NumStep_box, 'String', ud.NumStep );
      end;

      ud.NumStep = NStep( ud.t0, ud.tf, ud.deltat );

   case 'y0_edit'
      ud.y0 = cell( ud.numeqn, 1 );

      for i = 1:ud.numeqn
          ud.y0{i} = char( deblank( get( ud.Handle.y0_box(i), 'String' ) ));
      end

   case 'all_edit'
%      disp( 'Start of all-edit')
      f_order1( 'y_edit', num2str( h ) );
      f_order1( 't_edit', num2str( h ) );
      f_order1( 'f_edit', num2str( h ) );
      f_order1( 'pname_edit', num2str( h ) );
      f_order1( 'pvalue_edit', num2str( h ) );
      f_order1( 't0_edit', num2str( h ) );
      f_order1( 'y0_edit', num2str( h ) );
%      disp( 'End of all_edit' )

%      % Update all eqn window edit box strings. (Not needed in Unix because edit box
%      % callback is executed when focus leaves edit box.)
%      % fprintf(1, '\nStart of all_edit\n' );
%
%      for i = 1:ud.numeqn
%          ud.y{i} = char( deblank( get( ud.Handle.y_box(i), 'String' ) ));
%          ud.f{i} = char( deblank( get( ud.Handle.f_box(i), 'String' ) ));
%          ud.y0{i} = char( deblank( get( ud.Handle.y0_box(i), 'String' ) ));
%      end
%
%      ud.pname = cell( ud.nump, 1 );
%      ud.pvalue = cell( ud.nump, 1 );
%
%      for i = 1:ud.nump
%          ud.pname{i} = char( deblank( get( ud.Handle.pname_box(i), 'String' ) ));
%          ud.pvalue{i} = char( deblank( get( ud.Handle.pvalue_box(i), 'String' ) ));
%      end
%
%      ud.t0 = deblank( get( ud.Handle.t0_box(1), 'String' ) );
%      for i = 2:ud.numeqn
%         set( ud.Handle.t0_box(i), 'String', char( ud.t0 ) );
%      end

   end  % switch

   set( h, 'Userdata', ud );

% All edit boxes in control panels
case {  'tf_edit' 'deltat_edit' 'NumStep_edit' 'ymin_edit' ...
   'ymax_edit' }
   pud = get( gcbf, 'Userdata' );
   ud = get( pud.Handle.order1, 'UserData' );

   switch action
   case 'tf_edit'
      pud.tf = deblank( get( pud.Handle.tf_box, 'String' ) );
      pud.NumStep = NStep( ud.t0, pud.tf, pud.deltat );

      if ishandle( pud.Handle.NumStep_box )
         set( pud.Handle.NumStep_box, 'String', pud.NumStep );
      end

      %   set( ud.Handle.order1, 'UserData', ud );

   case 'deltat_edit'
      pud.deltat = deblank( get( pud.Handle.StepSize_box, 'String' ) );
      pud.NumStep = NStep( ud.t0, pud.tf, pud.deltat );
      set( pud.Handle.NumStep_box, 'String', pud.NumStep );
      %set( ud.Handle.order1, 'UserData', ud );

   case 'NumStep_edit'
      pud.NumStep = deblank( get( pud.Handle.NumStep_box, 'String' ) );

      x = str2num( pud.NumStep );
      if ~isint( x ) | x < 0
         w_message( ...
         [ 'ERROR: Enter a positive integer for the number of ' pud.method ' steps.'] );
      elseif x > 1e+4
         w_message( ...
         [ 'Are you sure you want ' num2str( x ) ' ' pud.method ' steps?'] );
      end;

      pud.deltat = DStep( ud.t0, pud.tf, pud.NumStep );
      set( pud.Handle.StepSize_box, 'String', pud.deltat );

   case 'ymin_edit'
      pud.ymin = deblank( get( pud.Handle.ymin_box, 'String' ) );

   case 'ymax_edit'
      pud.ymax = deblank( get( pud.Handle.ymax_box, 'String' ) );

   end  % switch

   set( gcbf, 'Userdata', pud );
%   UpdateOpenPanels ( ud );

otherwise
   fprintf( 'Error: f_order1 does not recognize the argument: %s\n', action );

end % switch

% end  f_order1

%===========================

function fout = prm2val( varargin )

% Substitutes values pvalue of parameters pname (cell arrays) into
% f.  Returns a character array.

% Can't blindly use strrep; e.g. 's = 6' turns 'cos' into 'co6'.

% subs( f, old, new ) requires that f be a scalar sym, tho' old
% and new can be cells.  An empty entry in old prevents
% substitution.

% Create symbolic parameter array; sym does not operate on cells
% Parameter value is empty if no value has been entered.

switch nargin
case 3
   f = varargin{1};
   pname = varargin{2};
   pvalue = varargin{3};
case 4
   f = varargin{1};
   pname = varargin{2};
   pvalue = varargin{3};
   y = varargin{4};
end  % case

nump = length(pvalue);
for i = 1:nump;
   pvalue{i} = sym( pvalue{i} );

   if isempty( pvalue{i} )
      % No value entered; do not substitute
      pname{i} = [];
   else
      pname{i} = sym( pname{i} );
   end

end

neqn = length( f );
fout = cell( neqn, 1 );
numbers = { '0' '1' '2' '3' '4' '5' '6' '7' '8' '9' };

for i = 1:neqn
   s = char( f{i} );

   if nump > 0

      % Kluge because subs turns sign() to +/- 1
      temp = deblank( s );
      s = strrep( temp, '[', '(' );
      temp = strrep( s, ']', ')' );
      s = strrep( temp, '{', '(' );
      temp = strrep( s, '}', ')' );
      signflag = 0;
      if findstr( 'sign(', temp )
         signflag = 1;
         s = strrep( temp, 'sign(', 'ngis*(' );
         syms ngis;
      end  % if

      temp = sym( s );
      s = subs( temp, pname, pvalue );
      % Preceding blows away sign(t - 1); also changes (P) to P.

      if isnumeric( s)
         % Kluge for purely numerical string
         fout{i} = num2str( s );
      else
         fout{i} = char( s );

         if signflag
            fout{i} = strrep( fout{i}, 'ngis*(', 'sign(' );

            % In case sign(P), say, became ngis*P, is ngis*
            % followed by a variable or a parameter?  Assume all
            % parameters and variable names of the form 'a..n..',
            % where a are letters, n are numbers; maybe no n in
            % name.
            start = findstr( 'ngis*', fout{i} );
            for k = start
               s = fout{i};
               len_f = length( fout{i} );
               after_ngis = s( (k + 5):len_f );
               for j = 1:neqn
                  name = char( y{j} );
                  len_name = length( name);

                  if len_name < length( after_ngis )
                     after_name = [ after_ngis(len_name + 1) ];
                  else
                     after_name = '*';
                  end  % if

                  found_name = strncmp( after_ngis, name, len_name );
                  letter_next = isletter( after_name );
                  number_next = ismember( after_name, numbers );

                  if found_name & ~letter_next & ~number_next
                     % ngis* is followed by a variable name;
                     % e.g., ngis*P2
                     if k > 1
                        head = s(1:(k - 1));
                     else
                        head = '';
                     end  % if

                     if  (k + 6 + len_name) < len_f
                        tail = s( (k + 6 + len_name):len_f );
                     else
                        tail = '';
                     end  % if
                     s = [ head 'sign(' name ')' tail ];
                  end  % if

               end  % for

%               Following code isn't needed because parameters
%               with values are replaced; e.g., sign(a), a =
%               0.12, becomes .12*sign, a disaster!
%
%               for j = 1:nump
%                  name = char( pname{j} )
%                  len_name = length( name);
%
%                  if len_name < length( after_ngis )
%                     after_name = [ after_ngis(len_name + 1) ];
%                  else
%                     after_name = '*';
%                  end  % if
%
%                  found_name = strncmp( after_ngis, name, len_name );
%                  letter_next = isletter( after_name );
%                  number_next = ismember( after_name, numbers );
%
%                  if ~isempty( deblank( char( pvalue{j} ) ) ) ...
%                     & found_name & ~letter_next & ~number_next
%                     % ngis* is followed by a parameter name
%                     if k > 1
%                        head = s(1:(k - 1));
%                     else
%                        head = '';
%                     end  % if
%
%                     if  (k + 6 + len_name) < len_f
%                        tail = s( (k + 6 + len_name):len_f );
%                     else
%                        tail = '';
%                     end  % if
%                     s = [ head 'sign(' name ')' tail ];
%                  end  % if
%
%               end  % for

               fout{i} = s;
            end  % for

         end  % if

      end

   else
%      fout{i} = char( sym( s ) );  % Don't ask; it works.
       fout{i} = f{i};
   end

end

% end prm2val

%===========================

%function strout = prm2val( ud )
%
%% Substitutes parameter values in ud.f, where ud is the UserData
%% structure defined in f_order1.  This function is local to
%% f_order1.  Can't blindly use strrep; e.g. 's = 6' turns 'cos' into 'co6'.
%
%strout = sym( ud.f );
%
%pname = ud.p1name;
%pval = ud.p1value;
%if ~isempty( num2str( pval ) )
%   % A parameter value has been entered. Substitute.
%   strout = subs( strout, sym( pname ), sym( pval ) );
%end
%
%pname = ud.p2name;
%pval = ud.p2value;
%if ~isempty( num2str( pval ) )
%   strout = subs( strout, sym( pname ), sym( pval ) );
%end
%
%strout = char( strout );
%
%% end prm2val
%
%===========================

function strout = myvectorize( strin )

% Vectorizes operations in the string strin: replaces *, /, and ^
% with .*, ./, and .^ .

% Strip user-entered vector operations, if present, then
% re-enter them.  Don't use with symbolic functions, even though
% the array operations are supposed to be sym/overloaded.
strout = strrep( strin, '.*', '*' );
strout = strrep( strout, './', '/' );
strout = strrep( strout, '.^', '^' );
strout = strrep( strout, '*', '.*' );
strout = strrep( strout, '/', './' );
strout = strrep( strout, '^', '.^' );

% end vectorize

%===========================

function strout = prettytitle( strin )

% prettytitle removes . from .*, etc. and removes * for
% pretty figure title output.

% Strip vector operations and '*'
strin = strrep( strin, '.*', '*' );
strin = strrep( strin, './', '/' );
strin = strrep( strin, '.^', '^' );
strin = strrep( strin, '*', ' ' );

% pad around +/-
strin = strrep( strin, '+', ' + ' );
strout = strrep( strin, '-', ' - ' );

% end prettytitle

%===========================

function deltat = DStep( t0, tf, n )

% DStep( t0, tf, n) returns (tf - t0)/n
% Input and output are all strings; checks for empty cases.

t0val = str2num( t0 );
tfval = str2num( tf );

if isempty( tfval ) | isempty( t0val )
   deltat = '';
elseif tfval <= t0val
   deltat = '';
else
   deltat = num2str( ( tfval - t0val  )/...
      str2num( n ) );
end;

% end DStep

%===========================

function NumStep = NStep( t0, tf, deltat )

% NStep( t0, tf, deltat) returns ceil( |tf - t0|/deltat )
% Input and output are all strings; checks for empty cases.

t0val = str2num( t0 );
tfval = str2num( tf );

if isempty( tfval ) | isempty( t0val )
   NumStep = '';
elseif tfval <= t0val
   NumStep = '';
else
   NumStep = num2str( abs( ceil( ( tfval - t0val  )/...
      str2num( deltat ) ) ) );
end;

% end NStep

%===========================

function ans = isint( x )

% isint( x ) returns 1 if x is an integer, 0 otherwise.

if class( x ) ~= 'double'
   ans = 0;
elseif ( x - floor( x) ) ~= 0
   ans = 0;
else
   ans = 1;
end;

% end isint

%===========================

function SolvDraw( ud, pud )

% SolvDraw( ud, pud ) draws figure of solution plot as requested from DELab
% ud is UserData from DELab equation window, pud from control
% panel window

set( gcbf, 'Pointer', 'watch' );

% Construct inline function and ode call arguments
fstring = char( prm2val( ud.f, ud.pname, ud.pvalue ) );
tname = char( ud.t );
yname = char( ud.y{1} );
yp = vectorize( inline( fstring, tname, yname ) );
y0cell = prm2val( ud.y0, ud.pname, ud.pvalue );
t0 = str2num( char( ud.t0 ) );
tf = str2num( char( pud.tf ) );
y0 = str2num( char( y0cell ) );

% Test of yp
try
   feval( yp, t0, y0 );
catch
   set( gcbf, 'Pointer', 'arrow' );
   errordlg( 'Faulty problem formulation.  Have parameters and initial conditions been assigned numeric values?', 'Error' );
   return;
end;

%set( ud.Handle.visdemo, 'Pointer', 'watch' );

% Set number of d field subdivisions
if ( str2num( pud.NumStep ) ) > 0
   numdraw = str2num( pud.NumStep );

   switch numdraw
   case ( numdraw == 1 | numdraw == 2 )
      numdraw = 10;
   case ( numdraw == 3 | numdraw == 4 )
      numdraw = 12;
   case ( numdraw >= 5 & numdraw <= 9 )
      numdraw = 2*numdraw;
   end;

else
   numdraw = 10;
end;

% Figure set up
if get( pud.Handle.NewFig_radio, 'Value' ) == 1 ...
   | pud.Handle.targetfig == -1
   % Start a new figure
   ymin = str2num( char( pud.ymin ) );
   ymax = str2num( char( pud.ymax ) );
   a = [ t0 tf ymin ymax ];
   pud.Handle.targetfig = figure;
   pud.Handle.HighAccuracy = -1;
   plotedit on;

   axis( a );
   axis manual;
%   x = get( pud.Handle.targetfig, 'Position' )
   hold on;
   dfld1( numdraw, a, yp, pud.style );
%   pud.visdemo.plotcount = 0;
   pud.visdemo.legendhandle = [];
   pud.visdemo.legendtext = {};
   plotcount = 0;
else
   % Use previous figure.
   figure( pud.Handle.targetfig );
   axis manual;
   hold on;
   plotcount = length( pud.visdemo.legendtext );
end;

% Solution plot title
titlestring = [ char( ud.y{1} ) ''' = ' char( ud.f{1} ) ];

for i = 1:length( ud.pvalue )
   if ~isempty( ud.pvalue{i} )
      titlestring = [ titlestring ', ' char( ud.pname{i} ) ' = ' char( ud.pvalue{i} ) ];
   end
end

feval( 'title', titlestring );
%s = prettytitle( fstring );
%title( [ 'd' yname '/d' tname ' = ' s ] );
h = feval( 'xlabel', [ tname '  -  Touch any key for next step.' ] );
set( h, 'Color', 'red' );
feval( 'ylabel', [ 'Approximate solution ' yname ] );

set( gcbf, 'Pointer', 'arrow' );

% Initial conditions if click on figure
if get( pud.Handle.UseEnteredIC_radio, 'Value' ) ~= 1
   % Click on figure
   set( pud.Handle.ClickText, 'Visible', 'on' );
   [t0 y0 ] = ginput(1);
   figure( pud.Handle.targetfig );
   set( pud.Handle.ClickText, 'Visible', 'off' );
end;

StringCell = cell(3, 1);
StringCell{1} = [ 'Starting at ' tname ' = ' num2str( t0 ) ', ' yname ' = ' num2str( y0 ) ','];
tspan = [ t0 tf ];

% Apply numerical method - add a stop check
if isempty( pud.method )
   w_message( 'WARNING: No method selected. Using ode45 by default.' );
   [t, y] = ode45( yp, tspan, y0 );
else
   step = str2num( pud.deltat );
   switch lower( char( pud.method ) )
   case 'euler'
      [t, y] = ode_euler( yp, tspan, y0, step );
   case 'heun'
      [t, y] = ode_heun( yp, tspan, y0, step );
   case 'rk4'
      [t, y] = ode_rk4( yp, tspan, y0, step );
   case 'ode23'
      [t, y] = ode23( yp, tspan, y0 );
   case 'ode45'
      options = odeset( 'Refine', 1 );
      [t, y] = ode45( yp, tspan, y0, options );
   case 'ode113'
      [t, y] = ode113( yp, tspan, y0 );
   case 'ode23s'
      [t, y] = ode23s( yp, tspan, y0 );
   case 'ode15s'
      [t, y] = ode15s( yp, tspan, y0 );
   end;
end;

% Plot solutions
%colorlist = [ 'b'; 'r'; 'c'; 'm'; 'y'; 'g' ];
colorlist = get(gca,'ColorOrder');
lcl = length( colorlist );

% High accuracy solution, if requested and not already plotted
if get( pud.Handle.high_check, 'Value' ) == 1 ...
   & pud.Handle.HighAccuracy < 0
   [texact yexact ] = ode45( yp, tspan, y0 );
   exactcolor = colorlist(lcl, :);
   hexact = plot( texact, yexact, 'Color', exactcolor );
   pud.Handle.HighAccuracy = hexact;
   plotcount = plotcount + 1;

%   if isempty( pud.visdemo.legendhandle )
%      pud.visdemo.legendhandle = [ hexact ];
%   else
      pud.visdemo.legendhandle = [ pud.visdemo.legendhandle hexact ];
%   end

   pud.visdemo.legendtext{plotcount} = [ 'High accuracy' ];
end;

% Plot approximate solution
plotcount = plotcount + 1;
approxcolor = colorlist(1 + mod( plotcount, lcl - 1 ), :);
happrox = feval ( 'plot', t(1), y(1,1), 'o', 'Color', approxcolor );
pud.visdemo.legendhandle = [ pud.visdemo.legendhandle happrox ];
pud.visdemo.legendtext{plotcount} = [ char( pud.method ) ' approx.' ];
legend( pud.visdemo.legendhandle, char( pud.visdemo.legendtext ) );

i = 1;
leny = length( t );
while i < leny
   %figure( pud.Handle.targetfig );
   pause;
%   feval ( 'plot', t(i + 1), y(i + 1,1), [ 'o' approxcolor ] );
%   feval ( 'plot', t(i:i + 1), y(i:i + 1,1), [ ':' approxcolor ] );
   feval ( 'plot', t(i + 1), y(i + 1,1), 'o', 'Color', approxcolor );
   feval ( 'plot', t(i:i + 1), y(i:i + 1,1), ':', 'Color', approxcolor );
   i = i + 1;
end;

h = xlabel( tname );
set( h, 'Color', 'black' );

%Report approximate solution value
StringCell{2} = [];
StringCell{3} = [ '   ' pud.method ' yields ' yname '(' num2str( t(leny) ) ') ~ ' num2str( y(leny) ) ];
hmsg = msgbox( StringCell, 'Approx. sol''n' );
pos = get( hmsg, 'Position' );
set( hmsg, 'Position', pos*diag( [ 0.1 1. 1. 1. ] ) );

%set( ud.Handle.visdemo, 'Pointer', 'arrow' );

% Update control panel window.
set( ud.Handle.visdemo, 'UserData', pud );

% end SolvDraw

%===========================

function h = dfld1( numpts, a, yp, style )

% dfld1( numpts, a, yp ) draws direction field for y' = yp on axes
% defined by array a.

t0 = a(1, 1);
tf = a(1, 2);
ymin = a(1, 3);
ymax = a(1, 4);

%numpts = min( 20, numpts );
delt = (tf - t0)/numpts;
dely = (ymax - ymin)/numpts;

%[t, y] = meshgrid( t0:delt:(tf - delt), ymin:dely:(ymax - dely) );
[t, y] = meshgrid( t0:delt:tf, ymin:dely:ymax );
grid = t + y.*sqrt(-1);
grid = grid(:);
t = t(:);
y = y(:);
[ ngridpts m ] = size( t );
yprime = feval( yp, t, y );
% Kluge for case yp = const
if length( yprime ) == 1
   yprime = yprime*ones( size( t ) );
end;
yprime = yprime(:);

% Scale for varied aspect ratio
aspect = abs( dely/delt );
z_t = zeros(ngridpts, 1);
z_y = zeros(ngridpts, 1);

for i = 1:ngridpts

   if ( abs( yprime(i) ) <= aspect )
      % "below" diagonal of delt-dely box
      z_t(i) = delt;
      z_y(i) = delt*yprime(i);
   else
      z_t(i) = abs( dely/yprime(i));
      z_y(i) = dely*sign(yprime(i));
   end;

end;

z = 0.9*(z_t + z_y*sqrt(-1));
z = z(:).';
%theta = atan( yprime );
%z = 0.9*(cos(theta)/abs(delt) + (sin(theta).*sqrt(-1))/abs(dely)).';
dash = [0 1].';
a = dash*z + ones(2,1)*grid.';
gray = .7*[ 1 1 1 ];

switch style
case 'lines'
   plot ( real(a), imag(a), '-y', 'Color', gray );
case 'arrows'
   h = quiver( t, y, z_t, z_y, 0.9 );
   set( h, 'Color', gray );
end %case

% end dfld

%===========================

function h = dfld2( numpts, a, yfunc, dummyt, style, size )

% dfld2( numpts, a, yfunc, dummyt ) draws the y vs. x direction field
% for the 2-D autonomous system x' = xp, y' = yp, yfunc = [ xp yp ],
% on axes defined by the array a using numpts horizontal and
% vertical spacing.  Style = 'lines' or 'arrows', size =
% 'uniform' or 'velocity' (-like).

xmin = a(1, 1);
xmax = a(1, 2);
ymin = a(1, 3);
ymax = a(1, 4);

delx = (xmax - xmin)/numpts;
dely = (ymax - ymin)/numpts;

gray = .7*[ 1 1 1 ];

[x, y] = meshgrid( xmin:delx:xmax, ymin:dely:ymax );
grid = x + y.*sqrt(-1);
grid = grid(:);
x = x(:);
y = y(:);
[ ngridpts m ] = size( x );
xprime = zeros(ngridpts, 1);
yprime = zeros(ngridpts, 1);

for i = 1:ngridpts
   z = feval( yfunc, dummyt, [ x(i); y(i) ] );
   xprime(i) = z(1);
   yprime(i) = z(2);
end

if strcmp( lower( size ), 'uniform' )
   % Scale for varied aspect ratio
   aspect = abs( dely/delx );
   z_x = zeros(ngridpts, 1);
   z_y = zeros(ngridpts, 1);

   for i = 1:ngridpts

      if xprime(i) ~= 0
         r = abs( yprime(i)/xprime(i) );
         if r <= aspect
            z_x(i) = delx*sign( xprime(i) );
            z_y(i) = delx*sign( yprime(i) )*r;
         else
            z_y(i) = dely*sign( yprime(i) );
            z_x(i) = dely*sign( xprime(i) )/r;
         end;
      else
         z_x(i) = 0;
         z_y(i) = dely*sign( yprime(i) );
      end

   end

   if strcmp( lower( style ), 'lines' )
      z = 0.9*(z_x  + z_y*sqrt(-1));
      z = z(:).';
      dash = [0 1].';
      a = dash*z + ones(2,1)*grid.';
      h = plot ( real(a), imag(a), '-', 'Color', gray );
   else
      % style == arrows
      h = quiver( x, y, z_x, z_y, 0.9 );
      set( h, 'Color', gray );
   end  % if

else
   % velocity-like

   if strcmp( lower( style ), 'lines' )
      yprime = yprime(:);
      xprime = xprime(:);
      scale = min( delx/max( abs( xprime )), dely/max( abs( yprime ) ) );
      xprime = scale*xprime;
      yprime = scale*yprime;

      z = xprime + yprime*sqrt(-1);
      z = z(:).';
      dash = [0 1].';
      a = dash*z + ones(2,1)*grid.';
      h = plot ( real(a), imag(a), '-', 'Color', gray );
   else
      % style == arrows
      h = quiver( x, y, xprime, yprime );
      set( h, 'Color', gray );
   end  % if

end % if

% end dfld2

%===========================

function solution = dsolve2cell( s, y )

% Converts output of dsolve or solve to cell vector of symbols.
% Solutions in cell vector are in the order corresponding to that
% of the cell vector y of variable names.

% dsolve can return a scalar expression of class sym, as from
% dsolve( 'Dy = y' ), an array of class sym, as from dsolve( 'Dy =
% (2/3)*y^(1/3)' ) (which has two solutions), or a structure, as
% from dsolve( 'Dy = z, Dz = -y' ).  DELab wants the output of
% dsolve to end up as a cell vector of symbols.

sclass = class( s );

switch sclass
case 'sym'
   len = length( s );
   if len == 1
      solution{1} = s;
   else
      solution = cell(len, 1);

      for i = 1:len
         solution{i} = s(i);
      end

      solution = solution(:);
   end

case 'struct'
% In case of multiple solutions, s is a structure whose field names
% are the variable names; e.g. s.F, s.R for steady states of the
% pred-prey eqns. The solution pairs are ( s.F{i}, s.R{i} ), i = 1,
% 2,..., and they must be listed in this order in the cell array
% solution.  When solution is constructed, want field names in s
% listed in same order as variable names in ud.y.

   realnames = fieldnames( s );
   names = y;

%   lenname = length( names )
%   solnution = cell(len, 1);
%   row = 0
%   for i = 1:lenname
%      soln = getfield( s, names{i} )
%
%      % In case each field is an array of symbols.
%      lensoln = length( soln )
%      for j = 1:slen
%         solution{row + j} = soln(j)
%      end
%      row = row + slen
%   end

   lenname = length( names );

   % kluge because solve won't find ss of SIRS
   if isfield( s, names{1} )
      fld = getfield( s, names{1} );
   else  % probably got wrong solution names
      solution = '';
      return;
   end  % if

   lenfield = length( fld );
   % Assuming all fields are same size ...
   solution = cell( lenname*lenfield, 1 );
   for f = 1:lenfield
      for n = 1:lenname
         symbol = getfield( s, names{n}, {f} );
         solution{ (f - 1)*lenname + n, 1 } = symbol;
      end
   end

   solution = solution(:);
otherwise
   warndlg( 'dsolve returned other than a sym or a struct.',...
      'Warning' );
   return;
end

% end dsolve2cell

%===========================

function value = getvalue( method )

% getvalue( method ) returns the index of the ivp method in the
% pop up menu in w_numivp.

string = lower( char( method ) );
switch string
case 'euler'
   value = 1;
case 'heun'
   value = 2;
case 'rk4'
   value = 3;
case 'ode23'
   value = 4;
case 'ode45'
   value = 5;
case 'ode113'
   value = 6;
case 'ode23s'
   value = 7;
case 'ode15s'
   value = 8;
end  % switch string

% end getvalue

%===========================copied from f_anlsoln

function [ solname, num ] = UniqueName( y, h0 );

% Constructs unique variable name y_num or y_vectornum from first
% entry in variable name cell array y, ideally using the figure
% number h0 of the calling window.  "Unique" is among base
% variables.

curvar = evalin( 'base', 'who' );

if length( y ) == 1
   vector = '_';
else
   vector = '_vec';
end;

num = h0;
solname = [ y{1} vector num2str( num ) ];
a = { [ '' solname '' ] };

while( ismember( a , curvar ) )
   num = num + 1;
   solname = [ y{1} vector num2str( num ) ];
   a = { [ '' solname '' ] };
end; % while

% end  UniqueName

%===========================

function UpdateOpenPanels ( ud )

% Updates UserData in any open control panels

if ud.Handle.numivp > 0
   set( ud.Handle.numivp, 'Userdata', ud )
elseif ud.Handle.visdemo > 0
   set( ud.Handle.visdemo, 'Userdata', ud )
elseif ud.Handle.plotvst > 0
   set( ud.Handle.plotvst, 'Userdata', ud )
elseif ud.Handle.phaseplane > 0
   set( ud.Handle.phaseplane, 'Userdata', ud )
end

if ud.Handle.order1 > 0
   set( ud.Handle.order1, 'UserData', ud )
end;

% end UpdateOpenPanels

%===========================

function g = makeinline( ud )

% Creates inline function to feed ode solver

% Replace parameters with their values.
fcell = prm2val( ud.f, ud.pname, ud.pvalue, ud.y );
% y0cell = prm2val( ud.y0, ud.pname, ud.pvalue );
% mycell = iscell( ud.y0 )
% mycellstr = iscellstr( ud.y0 )
y0cell = char( ud.y0 );

% Replace dependent variables names with yyyy(1), ...,
% yyyy(ud.neqn).
ycell = cell( ud.numeqn, 1 );
for i = 1:ud.numeqn
   s = [ 'yyyy(' num2str(i) ')' ];
   ycell{i, 1} = s;
end
fcell = prm2val( fcell, ud.y, ycell, ycell );

fchar = [ '[' char( fcell{1} ) ];
for i = 2:ud.numeqn
   fchar = [ fchar ';' char( fcell{i} ) ];
end
fchar = [ fchar ']' ];

g = vectorize( inline( fchar, 't', 'yyyy' ) );

%===========================

function output = badfunc( s )

% Issues error message and returns output = 1 if s contains 'sign'
% or other Matlab functions unacceptable to the symbolic toolbox.

output = 0;

if ~( isempty( findstr( lower( s ), 'sign' ) ) ...
   & isempty( findstr( lower( s ), 'ngis' ) ) )

   %should check sign is isolated; use isletter
   output = 1;
   bad = 'sign';
end  % if

if output
   bad = sprintf( '''%s''', bad );
   msg = [ 'Matlab''s symbolic toolbox does not recognize the function '...
      bad...
      ' even though it is acceptable'...
      ' for numerical or graphical applications.' ];
   errordlg( msg, 'Error' );
end  % if

% end badfunc

%===========================

function updatefiglist( pud )

% Updates the list of figure handles for figures containing plots.

[flag, fig] = figflag( '' );  % all existing plotted figures
fig = fig(:);
ud  = get( pud.Handle.order1, 'UserData' );

if isempty( fig )
   ud.Handle.plottedfigs = [];
else
   ud.Handle.plottedfigs = fig;
end;  % if

set( pud.Handle.order1, 'UserData', ud );

% end updatefiglist

%===========================

function [ outlist, ylist, outcount ] = vartoshow( numeqn, y, showvar );

% Lists indices, names, and count of variables to show on plot

% showvar is 0/1 vector of length numeqn of dependent variables
% named y to be plotted.  outlist is vector of indices of those
% variables, ylist cell of their (char) names, outcount the number
% of such variaables.

if numeqn > 1
   outlist = [];
   outcount = 0;
   ylist = '';

   for i = 1:numeqn

      if showvar{i} == 1
         outlist = [ outlist; i ];
         outcount = outcount + 1;
         ylist{outcount} = char( y{i} );

         % kluge: legend doesn't like one char string
         % when not plotting all variables.
         if length( y{i} ) == 1
            ylist{outcount} = [ ylist{outcount} ' ' ];
         end  % if

      end  % if

   end  % for

else
   outlist = [ 1 ];
   outcount = 1;
   ylist = char( y );
end  % if

% end vartoshow
