function fig = w_anlsoln( type, multsoln, solution,...
   ProbCell, SolnCell );

% w_anlsoln - window for analytic solution
% Displays problem string, solution string, and warning if
% possible multiple solutions for solution of DE or IVP.
% Displays equation string and steady state(s) for equilibria.
% Enables simplify and other solution manipulation.

% type = 'de', 'ivp', 'ss'
% ProbCell = problem string passed as cell array
% SolnCell = solution string passed as cell array

%load w_anlsoln
load w_delab

% Establish dimensions.
set( 0, 'units', 'character' );
size = get( 0, 'screensize' );
left = size(1, 1);
bottom = size(1, 2);
width = size(1, 3);
height = size(1, 4);
set( 0, 'units', 'pixels' );

set( 0, 'units', 'pixels' );
size = get( 0, 'screensize' );
hppc = size(1, 3)/width;     %horizontal pixels per inch
vppc = size(1, 4)/height;    %vertical pixels per inch
scale = diag( [ hppc vppc hppc vppc ] );

% Calculate size of display items in characters.
maxlinelength = 100;

% Find number of lines in box displaying problem, allowing for
% line breaks at spaces in order to keep longest line less than maxlinelength
% characters in length.

[ probwidth problines maxlinelength ] = TextLines( ProbCell, ...
   maxlinelength );

probwidth = probwidth + 10;
maxlinelength = max( maxlinelength, probwidth );

% Ditto for solution box.

[ solnwidth solnlines maxlinelength ] = TextLines( SolnCell, ...
   maxlinelength );
solnwidth = solnwidth + 10;
maxlinelength = max( maxlinelength, solnwidth );

buttonstring = 'Close';
buttonheight = 1.7;
buttonwidth = length( buttonstring ) + 4;

textboxheight = 1;
textboxwidth = length( 'has the single steady state' ) + 2;

hmargin = 2;
indent = 2*hmargin;
vmargin = 1;
figheight = vmargin + problines + vmargin + textboxheight + ...
   + vmargin + solnlines + vmargin + buttonheight + vmargin;
w1 = 2*(buttonwidth + hmargin) + hmargin;
w2 = max( [ probwidth solnwidth textboxwidth ] ) + 2*hmargin;
w3 = 1.4*length( 'Analytic solution' ) + 4 + 10;
w4 = length( 'Manipulate' ) + 4 + 10;
figwidth = max( [ w1 w2 w3 w4 ] );

x0 = left + 0.05*width;
y0 = bottom + 0.95*height - figheight;

switch type
case { 'de1', 'de2', 'ivp1', 'ivp2' }
   figname = 'Analytic solution';
   textline = 'is solved by';

case { 'ss1', 'ss2' }

    switch multsoln
    case 1
      figname = 'Steady state';
      textline = 'has the single steady state';
    otherwise
      figname = 'Steady states';
      textline = 'has the multiple steady states';
    end;

case 'numevalue'
   figname = 'Numerical eigenvalues';
   textline = 'has the eigenvalues and eigenvectors'

case 'num'
   figname = 'Numerical (approx.) solution';
   textline = 'has the approximate solution';
otherwise
   warndlg( 'w_anlsoln called with incorrect problem type',...
      'Warning' );
   return;
end;

%    'BackgroundColor',[ 0.8 0.8 0.8],... (use in children)
%   'Color',[0.8 0.8 0.8], ... (use in figure)
%MyColor = [ 0.9 0.9 0.9 ];
MyColor = [ 1 1 1 ];

h0 = figure('Color',MyColor, ...
    'Colormap',mat0, ...
    'MenuBar','none', ...
    'Name',figname, ...
    'NumberTitle','off', ...
    'PointerShapeCData',mat1, ...
    'Position',[x0 y0 figwidth figheight]*scale, ...
    'Tag','Fig2');

switch type
case { 'de1' 'de2' 'ivp1' 'ivp2' 'ss1' 'ss2' }
   udparent = get( gcbf, 'UserData' );
   ud.y = udparent.y;
   ud.f = udparent.f;
   ud.pname = udparent.pname;
   ud.pvalue = udparent.pvalue;
   neqn = length( ud.y );
   ud.y0 = udparent.y0;
   ud.t = udparent.t;
   ud.t0 = udparent.t0;

end  % switch

ud.type = type;
%ud.fstring = fstring;
ud.multsoln = multsoln;
ud.solution = solution;
ud.fig = h0;
set( h0, 'UserData', ud );

if ( multsoln > 1 )
   on_off = 'off';
else
   on_off = 'on';
end;

h1 = uimenu('Parent',h0, ...
    'Label','Manipulate' );
s1 = 'ud = get( gcf, ''UserData'' );';
s2 = 'f_anlsoln( ''simplify'', ud.type, ud.multsoln, ud.solution, ud.fig )';
h2 = uimenu('Parent',h1, ...
    'Label','Simplify', ...
    'Callback', [ s1 s2 ] );
s2 = 'f_anlsoln( ''simple'', ud.type, ud.multsoln, ud.solution, ud.fig )';
h2 = uimenu('Parent',h1, ...
    'Label','Simple', ...
    'Callback', [ s1 s2 ] );
%s2 = 'f_anlsoln( ''evaluate'', ud.type, ud.multsoln, ud.solution, ud.fig )';
%h2 = uimenu('Parent',h1, ...
%    'Label','Evaluate', ...
%    'Enable','off',...
%    'Callback', [ s1 s2 ] );
s2 = 'f_anlsoln( ''4digits'', ud.type, ud.multsoln, ud.solution, ud.fig )';
h2 = uimenu('Parent',h1, ...
    'Label','4 digit numbers', ...
    'Callback', [ s1 s2 ] );
s2 = 'f_anlsoln( ''8digits'', ud.type, ud.multsoln, ud.solution, ud.fig )';
h2 = uimenu('Parent',h1, ...
    'Label','8 digit numbers', ...
    'Callback', [ s1 s2 ] );
s2 = 'f_anlsoln( ''pretty'', ud.type, ud.multsoln, ud.solution, ud.fig )';
h2 = uimenu('Parent',h1, ...
    'Label','Pretty print', ...
    'Enable',on_off,...
    'Callback', [ s1 s2 ] );

h2 = uimenu('Parent',h1, ...
    'Enable',on_off,...
    'Label','Use in MATLAB as' );...
s2 = 'f_anlsoln( ''symbolic'', ud.type, ud.multsoln, ud.solution, ud.fig )';
h3 = uimenu('Parent',h2, ...
    'Label','Symbolic expression', ...
    'Enable',on_off,...
    'Callback', [ s1 s2 ] );
s2 = 'f_anlsoln( ''inline'', ud.type, ud.multsoln, ud.solution, ud.fig )';
h3 = uimenu('Parent',h2, ...
    'Label','In-line function', ...
    'Enable',on_off,...
    'Callback', [ s1 s2 ] );

h2 = uimenu('Parent',h1, ...
    'Enable',on_off,...
    'Label','Translate to' );...
s2 = 'f_anlsoln( ''latex'', ud.type, ud.multsoln, ud.solution, ud.fig )';
h3 = uimenu('Parent',h2, ...
    'Label','Latex', ...
    'Enable',on_off,...
    'Callback', [ s1 s2 ] );
s2 = 'f_anlsoln( ''ccode'', ud.type, ud.multsoln, ud.solution, ud.fig )';
h3 = uimenu('Parent',h2, ...
    'Label','C code', ...
    'Enable',on_off,...
    'Callback', [ s1 s2 ] );
s2 = 'f_anlsoln( ''fortran'', ud.type, ud.multsoln, ud.solution, ud.fig )';
h3 = uimenu('Parent',h2, ...
    'Label','Fortran', ...
    'Enable',on_off,...
    'Callback', [ s1 s2 ] );

%s1 = 'ud = get( gcf, ''UserData'' );';
%s2 = 'f_anlsoln( ''plot'', ud.type, ud.multsoln, ud.solution, ud.fig )';
%h1 = uimenu('Parent',h0, ...
%    'Label','Plot', ...
%    'Callback', [ s1 s2 ] );

%s1 = 'ud = get( gcf, ''UserData'' );';
%s2 = 'f_anlsoln( ''plot new'', ud.type, ud.multsoln, ud.solution, ud.fig )';
%h2 = uimenu('Parent',h1, ...
%    'Label','On new figure', ...
%    'Callback', [ s1 s2 ] );
%s2 = 'f_anlsoln( ''plot current'', ud.type, ud.multsoln, ud.solution, ud.fig )';
%h2 = uimenu('Parent',h1, ...
%    'Label','On current figure', ...
%    'Callback', [ s1 s2 ] );

h1 = uimenu('Parent',h0, ...
    'Label','Help' );
   h2 = uimenu('Parent',h1, ...
    'Callback','helpwin analytic', ...
    'Label','This window' );
   h2 = uimenu('Parent',h1, ...
    'Callback','helpwin contents', ...
    'Label','Contents' );
   h2 = uimenu('Parent',h1, ...
    'Callback','helpwin textbook', ...
    'Label','Textbook' );
   h2 = uimenu('Parent',h1, ...
    'Callback','helpwin about', ...
    'Label','About DELab' );

% Work from bottom up

% Print and close buttons
y = vmargin;
x = figwidth - hmargin - 2*(buttonwidth + hmargin);
h1 = uicontrol('Parent',h0, ...
    'Callback','GUIprint', ...
    'ListboxTop',0, ...
    'Position',[x y buttonwidth buttonheight]*scale, ...
    'String','Print', ...
    'TooltipString','Print this window' );

x = x + buttonwidth + hmargin;
h1 = uicontrol('Parent',h0, ...
    'Callback','close',...
    'ListboxTop',0, ...
    'Position',[x y buttonwidth buttonheight]*scale, ...
    'String','Close', ...
    'TooltipString','Close this window' );

% Solution
y = y + buttonheight + vmargin;
x = hmargin;

% To put solution string into workspace for symbolic manipulation,
% first define all variable names (ud.y, ud.t, ud.pname, C1, C2,
% etc.) as sym, then make global.  Then use code below to put
% symbol string into workspace.
%
% % Name for global symbolic solution string.
% ud = get( gcbf, 'Userdata' );
% globname = [ ud.y num2str( h0 ) ];
% % Make globname global.
% evalstring = [ 'global ' globname ];
% eval( evalstring );
% % Assign solution string to globname.
% evalstring = [ globname ' = SolnCell{1}' ];
% eval( evalstring );
% % Announce availability of this global variable.
% disp( [ 'Type global ' globname ] );

%global yy yyy;
%yy = SolnCell{1};
%yyy = 3;
h1 = uicontrol('Parent',h0, ...
    'BackgroundColor',MyColor,...
    'HorizontalAlignment','left',...
    'Position',[x y solnwidth solnlines]*scale, ...
    'String',SolnCell, ...
    'Style','text' );

% 'is solved by'
x = 0;
y = y + solnlines + vmargin;
h1 = uicontrol('Parent',h0, ...
    'BackgroundColor',MyColor,...
    'HorizontalAlignment','center',...
    'Position',[x y figwidth textboxheight]*scale, ...
    'String',textline, ...
    'Style','text' );

% Problem
x = hmargin;
y = y + textboxheight + vmargin;
h1 = uicontrol('Parent',h0, ...
    'BackgroundColor',MyColor,...
    'HorizontalAlignment','left',...
    'Position',[x y probwidth problines]*scale, ...
    'String',ProbCell, ...
    'Style','text' );

if nargout > 0, fig = h0; end

% ====================================

function [ width, lines, maxlinelength ] = TextLines( TextCell, ...
   maxlinelength );

% TextLines( TextCell, maxlinelength ) find total number of lines lines
% in TextCell, a cell array of strings of text, when each string is wrapped
% at a blank in order to keep the right margin within maxlinelength, if
% possible. Returns width, width of longest line, and maxlinelength,
% enlarged if needed to accommodate excessively long strings without blanks.

[rows m] = size( TextCell );
width = 0;
extralines = 0;

for i = 1:rows
   len = length( TextCell{i} );

   if len <= maxlinelength
      % Line isn't long enough to wrap.

      if width < len  width = len; end

   else
      % Wrap line at blanks nearest to right margin.
      % Pad array of possible breaks with start and end points.
      s = findstr( TextCell{i}, ' ' );
      breaks = [ 0 s len ];
      lenbreaks = length( breaks );

      % Index to break points in breaks
      oldindex = 1;
      newindex = 2;

      while newindex <= lenbreaks
         % Start at newindex in search for next break index
         % after oldindex.

         while ( breaks(newindex) - breaks(oldindex) < maxlinelength )...
            & ( newindex < lenbreaks )
            % Try next break point.
            newindex = newindex + 1;
         end;  % while

         if newindex - oldindex > 1
            % A break at newindex - 1 will break the
            % line before maxlinelength
            newindex = newindex - 1;
%          else
%             % newindex - oldindex == 1 and maxlinelength must expand
%             % to avoid a break at other than a blank space.
         end % if

         width = max( width, ...
            breaks(newindex) - breaks(oldindex) );
         maxlinelength = max( maxlinelength, width );
         extralines = extralines + 1;
         oldindex = newindex;
         newindex = oldindex + 1;
      end % while

   end; % if

end; % for

lines = rows + extralines;
% end TextLines


% Possible menu bar items - all via callbacks to f_anlsoln.
%
% Plot
%    Plot on current figure
%    Plot on new figure
%       (Both open dialog to get first (if needed), last t, any undefined parameters
%       or constants. Use sym/ezplot?)
%    Linearize (about ss - choose which one)
